@@ -28,6 +28,7 @@ from Orange.misc.environ import data_dir
 from Orange.widgets import settings, gui
 from Orange.widgets.utils.signals import Output
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.widget import OWWidget, Msg
 
 
@@ -43,6 +44,15 @@ def ensure_local(index_url, file_path, local_cache_path,
     return localfiles.localpath_download(*file_path, callback=progress_advance)
 
 
+def list_remote(server: str) -> Dict[Tuple[str, ...], dict]:
+    client = ServerFiles(server)
+    return client.allinfo()
+
+
+def list_local(path: str) -> Dict[Tuple[str, ...], dict]:
+    return LocalFiles(path).allinfo()
+
+
 def format_exception(error):
     # type: (BaseException) -> str
     return "\n".join(traceback.format_exception_only(type(error), error))
@@ -134,7 +144,8 @@ class OWDataSets(OWWidget):
     icon = "icons/DataSets.svg"
     priority = 20
     replaces = ["orangecontrib.prototypes.widgets.owdatasets.OWDataSets"]
-    keywords = ["online"]
+    keywords = ["online", "data sets", 'shujuji', 'zaixian']
+    category = "Data"
 
     want_control_area = False
 
@@ -232,6 +243,8 @@ class OWDataSets(OWWidget):
         self.splitter.addWidget(self.view)
         self.splitter.addWidget(box)
 
+        self.info.set_output_summary(self.info.NoOutput)
+
         self.splitter.setSizes([300, 200])
         self.splitter.splitterMoved.connect(
             lambda:
@@ -253,7 +266,7 @@ class OWDataSets(OWWidget):
         self.setStatusMessage("Initializing")
 
         self._executor = ThreadPoolExecutor(max_workers=1)
-        f = self._executor.submit(self.list_remote)
+        f = self._executor.submit(list_remote, self.INDEX_URL)
         w = FutureWatcher(f, parent=self)
         w.done.connect(self.__set_index)
 
@@ -346,7 +359,7 @@ class OWDataSets(OWWidget):
         assert f.done()
         self.setBlocking(False)
         self.setStatusMessage("")
-        self.allinfo_local = self.list_local()
+        self.allinfo_local = list_local(self.local_cache_path)
 
         try:
             self.allinfo_remote = f.result()
@@ -382,7 +395,7 @@ class OWDataSets(OWWidget):
 
     def __update_cached_state(self):
         model = self.view.model().sourceModel()
-        localinfo = self.list_local()
+        localinfo = list_local(self.local_cache_path)
         assert isinstance(model, QStandardItemModel)
         allinfo = []
         for i in range(model.rowCount()):
@@ -535,9 +548,12 @@ class OWDataSets(OWWidget):
     def load_and_output(self, path):
         if path is None:
             self.Outputs.data.send(None)
+            self.info.set_output_summary(self.info.NoOutput)
         else:
             data = self.load_data(path)
             self.Outputs.data.send(data)
+            self.info.set_output_summary(len(data),
+                                         format_summary_details(data))
 
         self.current_output = path
         self.__update_cached_state()
@@ -546,15 +562,6 @@ class OWDataSets(OWWidget):
     def load_data(path):
         return Orange.data.Table(path)
 
-    def list_remote(self):
-        # type: () -> Dict[Tuple[str, ...], dict]
-        client = ServerFiles(server=self.INDEX_URL)
-        return client.allinfo()
-
-    def list_local(self):
-        # type: () -> Dict[Tuple[str, ...], dict]
-        return LocalFiles(self.local_cache_path).allinfo()
-
 
 class FutureWatcher(QObject):
     done = Signal(object)
