@@ -7,14 +7,14 @@ TODO:
 """
 
 import datetime
-import locale
 from enum import IntEnum
+from itertools import chain
 from typing import Any, Optional, Tuple, List
 
 import numpy as np
 import scipy.stats as ss
 import scipy.sparse as sp
-from AnyQt.QtCore import Qt, QSize, QRectF, QVariant, QModelIndex, pyqtSlot, \
+from AnyQt.QtCore import Qt, QSize, QRectF, QModelIndex, pyqtSlot, \
     QRegExp, QItemSelection, QItemSelectionRange, QItemSelectionModel
 from AnyQt.QtGui import QPainter, QColor
 from AnyQt.QtWidgets import QStyledItemDelegate, QGraphicsScene, QTableView, \
@@ -25,11 +25,12 @@ from Orange.data import Table, StringVariable, DiscreteVariable, \
     ContinuousVariable, TimeVariable, Domain, Variable
 from Orange.widgets import widget, gui
 from Orange.widgets.data.utils.histogram import Histogram
-from Orange.widgets.report import plural
-from Orange.widgets.settings import ContextSetting, DomainContextHandler
+from Orange.widgets.settings import Setting, ContextSetting, \
+    DomainContextHandler
 from Orange.widgets.utils.itemmodels import DomainModel, AbstractSortTableModel
 from Orange.widgets.utils.signals import Input, Output
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 
 
 def _categorical_entropy(x):
@@ -40,6 +41,14 @@ def _categorical_entropy(x):
     return np.fromiter((ss.entropy(pk) for pk in p), dtype=np.float64)
 
 
+def coefficient_of_variation(x: np.ndarray) -> np.ndarray:
+    mu = ut.nanmean(x, axis=0)
+    mask = ~np.isclose(mu, 0, atol=1e-12)
+    result = np.full_like(mu, fill_value=np.inf)
+    result[mask] = np.sqrt(ut.nanvar(x, axis=0)[mask]) / mu[mask]
+    return result
+
+
 def format_time_diff(start, end, round_up_after=2):
     """Return an approximate human readable time difference between two dates.
 
@@ -108,8 +117,8 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
             return {self.ICON: '',
                     self.NAME: '名称',
                     self.DISTRIBUTION: '分布',
-                    self.CENTER: '中心',
-                    self.DISPERSION: '分散',
+                    self.CENTER: '集中趋势',
+                    self.DISPERSION: '离散',
                     self.MIN: '最小.',
                     self.MAX: '最大.',
                     self.MISSING: '缺失',
@@ -140,6 +149,10 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
         self.n_attributes = self.n_instances = 0
 
         self.__attributes = self.__class_vars = self.__metas = None
+        # sets of variables for fast membership tests
+        self.__attributes_set = set()
+        self.__class_vars_set = set()
+        self.__metas_set = set()
         self.__distributions_cache = {}
 
         no_data = np.array([])
@@ -167,7 +180,9 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
         # while we need a 2d array, which `_Y` provides
         self.__class_vars = self.__filter_attributes(domain.class_vars, self.table._Y)  # pylint: disable=protected-access
         self.__metas = self.__filter_attributes(domain.metas, self.table.metas)
-
+        self.__attributes_set = set(self.__metas[0])
+        self.__class_vars_set = set(self.__class_vars[0])
+        self.__metas_set = set(self.__metas[0])
         self.n_attributes = len(self.variables)
         self.n_instances = len(data)
 
@@ -182,6 +197,9 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
         self.__attributes = (np.array([]), np.array([]))
         self.__class_vars = (np.array([]), np.array([]))
         self.__metas = (np.array([]), np.array([]))
+        self.__attributes_set = set()
+        self.__class_vars_set = set()
+        self.__metas_set = set()
         self.__distributions_cache.clear()
         self.endResetModel()
 
@@ -229,7 +247,7 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
         self._dispersion = self.__compute_stat(
             matrices,
             discrete_f=_categorical_entropy,
-            continuous_f=lambda x: np.sqrt(ut.nanvar(x, axis=0)) / ut.nanmean(x, axis=0),
+            continuous_f=coefficient_of_variation,
         )
         self._missing = self.__compute_stat(
             matrices,
@@ -237,6 +255,7 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
             continuous_f=lambda x: ut.countnans(x, axis=0),
             string_f=lambda x: (x == StringVariable.Unknown).sum(axis=0),
             time_f=lambda x: ut.countnans(x, axis=0),
+            default_val=len(matrices[0]) if matrices else 0
         )
         self._max = self.__compute_stat(
             matrices,
@@ -314,16 +333,6 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
         if not matrices:
             return np.array([])
 
-        def _to_float(data):
-            if not np.issubdtype(data.dtype, np.number):
-                data = data.astype(np.float64)
-            return data
-
-        def _to_object(data):
-            if data.dtype is not np.object:
-                data = data.astype(np.object)
-            return data
-
         results = []
         for variables, x in matrices:
             result = np.full(len(variables), default_val)
@@ -331,23 +340,28 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
             # While the following caching and checks are messy, the indexing
             # turns out to be a bottleneck for large datasets, so a single
             # indexing operation improves performance
-            disc_idx, cont_idx, time_idx, str_idx = self._attr_indices(variables)
-            if discrete_f:
-                x_ = x[:, disc_idx]
-                if x_.size:
-                    result[disc_idx] = discrete_f(_to_float(x_))
-            if continuous_f:
-                x_ = x[:, cont_idx]
-                if x_.size:
-                    result[cont_idx] = continuous_f(_to_float(x_))
-            if time_f:
-                x_ = x[:, time_idx]
-                if x_.size:
-                    result[time_idx] = time_f(_to_float(x_))
+            *idxs, str_idx = self._attr_indices(variables)
+            for func, idx in zip((discrete_f, continuous_f, time_f), idxs):
+                idx = np.array(idx)
+                if func and idx.size:
+                    x_ = x[:, idx]
+                    if x_.size:
+                        if not np.issubdtype(x_.dtype, np.number):
+                            x_ = x_.astype(np.float64)
+                        try:
+                            finites = np.isfinite(x_)
+                        except TypeError:
+                            result[idx] = func(x_)
+                        else:
+                            mask = np.any(finites, axis=0)
+                            if np.any(mask):
+                                result[idx[mask]] = func(x_[:, mask])
             if string_f:
                 x_ = x[:, str_idx]
                 if x_.size:
-                    result[str_idx] = string_f(_to_object(x_))
+                    if x_.dtype is not np.object:
+                        x_ = x_.astype(np.object)
+                    result[str_idx] = string_f(x_)
 
             results.append(result)
 
@@ -465,44 +479,51 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
 
     def data(self, index, role):
         # type: (QModelIndex, Qt.ItemDataRole) -> Any
-        # Text formatting for various data simply requires a lot of branches.
-        # This is much better than overengineering various formatters...
-        # pylint: disable=too-many-branches
-
-        if not index.isValid():
+        def background():
+            if attribute in self.__attributes_set:
+                return self.COLOR_FOR_ROLE[self.ATTRIBUTE]
+            if attribute in self.__metas_set:
+                return self.COLOR_FOR_ROLE[self.META]
+            if attribute in self.__class_vars_set:
+                return self.COLOR_FOR_ROLE[self.CLASS_VAR]
             return None
 
-        row, column = self.mapToSourceRows(index.row()), index.column()
-        # Make sure we're not out of range
-        if not 0 <= row <= self.n_attributes:
-            return QVariant()
-
-        attribute = self.variables[row]
-
-        if role == Qt.BackgroundRole:
-            if attribute in self.domain.attributes:
-                return self.COLOR_FOR_ROLE[self.ATTRIBUTE]
-            elif attribute in self.domain.metas:
-                return self.COLOR_FOR_ROLE[self.META]
-            elif attribute in self.domain.class_vars:
-                return self.COLOR_FOR_ROLE[self.CLASS_VAR]
-
-        elif role == Qt.TextAlignmentRole:
+        def text_alignment():
             if column == self.Columns.NAME:
                 return Qt.AlignLeft | Qt.AlignVCenter
             return Qt.AlignRight | Qt.AlignVCenter
 
-        output = None
-
-        if column == self.Columns.ICON:
-            if role == Qt.DecorationRole:
+        def decoration():
+            if column == self.Columns.ICON:
                 return gui.attributeIconDict[attribute]
-        elif column == self.Columns.NAME:
-            if role == Qt.DisplayRole:
-                output = attribute.name
-        elif column == self.Columns.DISTRIBUTION:
-            if role == Qt.DisplayRole:
-                if isinstance(attribute, (DiscreteVariable, ContinuousVariable)):
+            return None
+
+        def display():
+            # pylint: disable=too-many-branches
+            def format_zeros(str_val):
+                """Zeros should be handled separately as they cannot be negative."""
+                if float(str_val) == 0:
+                    num_decimals = min(self.variables[row].number_of_decimals, 2)
+                    str_val = f"{0:.{num_decimals}f}"
+                return str_val
+
+            def render_value(value):
+                if np.isnan(value):
+                    return ""
+                if np.isinf(value):
+                    return "∞"
+
+                str_val = attribute.str_val(value)
+                if attribute.is_continuous and not attribute.is_time:
+                    str_val = format_zeros(str_val)
+
+                return str_val
+
+            if column == self.Columns.NAME:
+                return attribute.name
+            elif column == self.Columns.DISTRIBUTION:
+                if isinstance(attribute,
+                              (DiscreteVariable, ContinuousVariable)):
                     if row not in self.__distributions_cache:
                         scene = QGraphicsScene(parent=self)
                         histogram = Histogram(
@@ -515,60 +536,43 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
                         scene.addItem(histogram)
                         self.__distributions_cache[row] = scene
                     return self.__distributions_cache[row]
-        elif column == self.Columns.CENTER:
-            if role == Qt.DisplayRole:
-                if isinstance(attribute, DiscreteVariable):
-                    output = self._center[row]
-                    if not np.isnan(output):
-                        output = attribute.str_val(self._center[row])
-                elif isinstance(attribute, TimeVariable):
-                    output = attribute.str_val(self._center[row])
-                else:
-                    output = self._center[row]
-        elif column == self.Columns.DISPERSION:
-            if role == Qt.DisplayRole:
+            elif column == self.Columns.CENTER:
+                return render_value(self._center[row])
+            elif column == self.Columns.DISPERSION:
                 if isinstance(attribute, TimeVariable):
-                    output = format_time_diff(self._min[row], self._max[row])
+                    return format_time_diff(self._min[row], self._max[row])
+                elif isinstance(attribute, DiscreteVariable):
+                    return "%.3g" % self._dispersion[row]
                 else:
-                    output = self._dispersion[row]
-        elif column == self.Columns.MIN:
-            if role == Qt.DisplayRole:
-                if isinstance(attribute, DiscreteVariable):
-                    if attribute.ordered:
-                        output = attribute.str_val(self._min[row])
-                elif isinstance(attribute, TimeVariable):
-                    output = attribute.str_val(self._min[row])
-                else:
-                    output = self._min[row]
-        elif column == self.Columns.MAX:
-            if role == Qt.DisplayRole:
-                if isinstance(attribute, DiscreteVariable):
-                    if attribute.ordered:
-                        output = attribute.str_val(self._max[row])
-                elif isinstance(attribute, TimeVariable):
-                    output = attribute.str_val(self._max[row])
-                else:
-                    output = self._max[row]
-        elif column == self.Columns.MISSING:
-            if role == Qt.DisplayRole:
-                output = '%d (%d%%)' % (
+                    return render_value(self._dispersion[row])
+            elif column == self.Columns.MIN:
+                if not isinstance(attribute, DiscreteVariable):
+                    return render_value(self._min[row])
+            elif column == self.Columns.MAX:
+                if not isinstance(attribute, DiscreteVariable):
+                    return render_value(self._max[row])
+            elif column == self.Columns.MISSING:
+                return '%d (%d%%)' % (
                     self._missing[row],
                     100 * self._missing[row] / self.n_instances
                 )
+            return None
 
-        # Consistently format the text inside the table cells
-        # The easiest way to check for NaN is to compare with itself
-        if output != output:  # pylint: disable=comparison-with-itself
-            output = ''
-        # Format ∞ properly
-        elif output in (np.inf, -np.inf):
-            output = '%s∞' % ['', '-'][output < 0]
-        elif isinstance(output, int):
-            output = locale.format_string('%d', output, grouping=True)
-        elif isinstance(output, float):
-            output = locale.format_string('%.2f', output, grouping=True)
+        roles = {Qt.BackgroundRole: background,
+                 Qt.TextAlignmentRole: text_alignment,
+                 Qt.DecorationRole: decoration,
+                 Qt.DisplayRole: display}
 
-        return output
+        if not index.isValid() or role not in roles:
+            return None
+
+        row, column = self.mapToSourceRows(index.row()), index.column()
+        # Make sure we're not out of range
+        if not 0 <= row <= self.n_attributes:
+            return None
+
+        attribute = self.variables[row]
+        return roles[role]()
 
     def rowCount(self, parent=QModelIndex()):
         return 0 if parent.isValid() else self.n_attributes
@@ -698,6 +702,8 @@ class OWFeatureStatistics(widget.OWWidget):
     name = '特征统计(Feature Statistics)'
     description = '显示数据特征的基本统计信息。'
     icon = 'icons/FeatureStatistics.svg'
+    category = "Data"
+    keywords = ['tezheng', 'tongji', 'tezhengtongji']
 
     class Inputs:
         data = Input('数据(Data)', Table, default=True, replaces=['Data'])
@@ -706,32 +712,24 @@ class OWFeatureStatistics(widget.OWWidget):
         reduced_data = Output('选中的数据(Reduced Data)', Table, default=True, replaces=['Reduced Data'])
         statistics = Output('统计(Statistics)', Table, replaces=['Statistics'])
 
-    want_main_area = True
+    want_control_area = False
     buttons_area_orientation = Qt.Vertical
 
     settingsHandler = DomainContextHandler()
+    settings_version = 2
 
-    auto_commit = ContextSetting(True)
+    auto_commit = Setting(True)
     color_var = ContextSetting(None)  # type: Optional[Variable]
     # filter_string = ContextSetting('')
 
-    sorting = ContextSetting((0, Qt.DescendingOrder))
-    selected_rows = ContextSetting([])
+    sorting = Setting((0, Qt.DescendingOrder))
+    selected_vars = ContextSetting([], schema_only=True)
 
     def __init__(self):
         super().__init__()
 
         self.data = None  # type: Optional[Table]
 
-        # Information panel
-        info_box = gui.vBox(self.controlArea, '信息')
-        info_box.setMinimumWidth(200)
-        self.info_summary = gui.widgetLabel(info_box, wordWrap=True)
-        self.info_attr = gui.widgetLabel(info_box, wordWrap=True)
-        self.info_class = gui.widgetLabel(info_box, wordWrap=True)
-        self.info_meta = gui.widgetLabel(info_box, wordWrap=True)
-        self.set_info()
-
         # TODO: Implement filtering on the model
         # filter_box = gui.vBox(self.controlArea, 'Filter')
         # self.filter_text = gui.lineEdit(
@@ -742,19 +740,26 @@ class OWFeatureStatistics(widget.OWWidget):
         # shortcut = QShortcut(QKeySequence('Ctrl+f'), self, self.filter_text.setFocus)
         # shortcut.setWhatsThis('Filter variables by name')
 
+        box = gui.hBox(None, box=False)
+        box.setContentsMargins(0, 0, 0, 0)
+
         self.color_var_model = DomainModel(
             valid_types=(ContinuousVariable, DiscreteVariable),
             placeholder='无',
         )
-        box = gui.vBox(self.controlArea, '直方图(Histogram)')
         self.cb_color_var = gui.comboBox(
             box, master=self, value='color_var', model=self.color_var_model,
-            label='颜色:', orientation=Qt.Horizontal,
+            label='颜色:', orientation=Qt.Horizontal, contentsLength=13,
+            searchable=True
         )
         self.cb_color_var.activated.connect(self.__color_var_changed)
 
-        gui.rubber(self.controlArea)
-        gui.auto_send(self.buttonsArea, self, "auto_commit")
+        gui.rubber(box)
+        gui.auto_send(box, self, "auto_commit", box=None)
+        self.mainArea.layout().addWidget(box)
+
+        self.info.set_input_summary(self.info.NoInput)
+        self.info.set_output_summary(self.info.NoOutput)
 
         # Main area
         self.model = FeatureStatisticsTableModel(parent=self)
@@ -783,7 +788,7 @@ class OWFeatureStatistics(widget.OWWidget):
     def set_data(self, data):
         # Clear outputs and reset widget state
         self.closeContext()
-        self.selected_rows = []
+        self.selected_vars = []
         self.model.resetSorting()
         self.Outputs.reduced_data.send(None)
         self.Outputs.statistics.send(None)
@@ -792,11 +797,14 @@ class OWFeatureStatistics(widget.OWWidget):
         self.data = data
 
         if data is not None:
+            self.info.set_input_summary(len(data),
+                                        format_summary_details(data))
             self.color_var_model.set_domain(data.domain)
             self.color_var = None
             if self.data.domain.class_vars:
                 self.color_var = self.data.domain.class_vars[0]
         else:
+            self.info.set_input_summary(self.info.NoInput)
             self.color_var_model.set_domain(None)
             self.color_var = None
         self.model.set_data(data)
@@ -807,17 +815,16 @@ class OWFeatureStatistics(widget.OWWidget):
         # self._filter_table_variables()
         self.__color_var_changed()
 
-        self.set_info()
         self.commit()
 
     def __restore_selection(self):
         """Restore the selection on the table view from saved settings."""
         selection_model = self.table_view.selectionModel()
         selection = QItemSelection()
-        # self.selected_rows can be list or numpy.array, thus
-        # pylint: disable=len-as-condition
-        if len(self.selected_rows):
-            for row in self.model.mapFromSourceRows(self.selected_rows):
+        if self.selected_vars:
+            var_indices = {var: i for i, var in enumerate(self.model.variables)}
+            selected_indices = [var_indices[var] for var in self.selected_vars]
+            for row in self.model.mapFromSourceRows(selected_indices):
                 selection.append(QItemSelectionRange(
                     self.model.index(row, 0),
                     self.model.index(row, self.model.columnCount() - 1)
@@ -843,79 +850,24 @@ class OWFeatureStatistics(widget.OWWidget):
         if self.model is not None:
             self.model.set_target_var(self.color_var)
 
-    def _format_variables_string(self, variables):
-        agg = []
-        for var_type_name, var_type in [
-                ('个分类', DiscreteVariable),
-                ('个数值', ContinuousVariable),
-                ('个时间', TimeVariable),
-                ('个字符串', StringVariable)
-        ]:
-            # Disable pylint here because a `TimeVariable` is also a
-            # `ContinuousVariable`, and should be labelled as such. That is why
-            # it is necessary to check the type this way instead of using
-            # `isinstance`, which would fail in the above case
-            var_type_list = [v for v in variables if type(v) is var_type]  # pylint: disable=unidiomatic-typecheck
-            if var_type_list:
-                shown = var_type in self.model.HIDDEN_VAR_TYPES
-                agg.append((
-                    '%d %s%s' % (len(var_type_list), var_type_name, ['', '(未显示)'][shown]),
-                    len(var_type_list)
-                ))
-
-        if not agg:
-            return '没有变量'
-
-        attrs, counts = list(zip(*agg))
-        if len(attrs) > 1:
-            var_string = ', '.join(attrs[:-1]) + ' 和 ' + attrs[-1]
-        else:
-            var_string = attrs[0]
-        return plural('%s变量' % var_string, sum(counts))
-
-    def set_info(self):
-        if self.data is not None:
-            self.info_summary.setText('<b>%s</b>包含%s个具有%s特征的实例' % (
-                self.data.name,
-                plural('{number}', self.model.n_instances),
-                plural('{number}', self.model.n_attributes)
-                
-            ))
-
-            self.info_attr.setText(
-                '<b>属性:</b><br>%s' %
-                self._format_variables_string(self.data.domain.attributes)
-            )
-            self.info_class.setText(
-                '<b>分类变量:</b><br>%s' %
-                self._format_variables_string(self.data.domain.class_vars)
-            )
-            self.info_meta.setText(
-                '<b>元变量:</b><br>%s' %
-                self._format_variables_string(self.data.domain.metas)
-            )
-        else:
-            self.info_summary.setText('没有输入数据。')
-            self.info_attr.setText('')
-            self.info_class.setText('')
-            self.info_meta.setText('')
-
     def on_select(self):
-        self.selected_rows = list(self.model.mapToSourceRows([
+        selection_indices = list(self.model.mapToSourceRows([
             i.row() for i in self.table_view.selectionModel().selectedRows()
         ]))
+        self.selected_vars = list(self.model.variables[selection_indices])
         self.commit()
 
     def commit(self):
-        # self.selected_rows can be list or numpy.array, thus
-        # pylint: disable=len-as-condition
-        if not len(self.selected_rows):
+        if not self.selected_vars:
+            self.info.set_output_summary(self.info.NoOutput)
             self.Outputs.reduced_data.send(None)
             self.Outputs.statistics.send(None)
             return
 
         # Send a table with only selected columns to output
-        variables = self.model.variables[self.selected_rows]
+        variables = self.selected_vars
+        self.info.set_output_summary(len(self.data[:, variables]),
+                                     format_summary_details(self.data[:, variables]))
         self.Outputs.reduced_data.send(self.data[:, variables])
 
         # Send the statistics of the selected variables to ouput
@@ -930,7 +882,28 @@ class OWFeatureStatistics(widget.OWWidget):
         self.Outputs.statistics.send(statistics)
 
     def send_report(self):
-        pass
+        view = self.table_view
+        self.report_table(view)
+
+    @classmethod
+    def migrate_context(cls, context, version):
+        if not version or version < 2:
+            selected_rows = context.values.pop("selected_rows", None)
+            if not selected_rows:
+                selected_vars = []
+            else:
+                # This assumes that dict was saved by Python >= 3.6 so dict is
+                # ordered; if not, context hasn't had worked anyway.
+                all_vars = [
+                    (var, tpe)
+                    for (var, tpe) in chain(context.attributes.items(),
+                                            context.metas.items())
+                    # it would be nicer to use cls.HIDDEN_VAR_TYPES, but there
+                    # is no suitable conversion function, and StringVariable (3)
+                    # was the only hidden var when settings_version < 2, so:
+                    if tpe != 3]
+                selected_vars = [all_vars[i] for i in selected_rows]
+            context.values["selected_vars"] = selected_vars, -3
 
 
 if __name__ == '__main__':  # pragma: no cover
