@@ -4,8 +4,10 @@ from itertools import chain, product
 import numpy as np
 
 from AnyQt.QtCore import Qt, QModelIndex, pyqtSignal as Signal
-from AnyQt.QtWidgets import QWidget, QLabel, QComboBox, QPushButton, \
-    QVBoxLayout, QHBoxLayout
+from AnyQt.QtWidgets import (
+    QWidget, QLabel, QPushButton, QVBoxLayout, QHBoxLayout
+)
+from orangewidget.utils.combobox import ComboBoxSearch
 
 import Orange
 from Orange.data import StringVariable, ContinuousVariable, Variable
@@ -16,10 +18,12 @@ from Orange.widgets.utils import vartype
 from Orange.widgets.utils.itemmodels import DomainModel
 from Orange.widgets.utils.sql import check_sql_input
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_multiple_summaries, \
+    format_summary_details
 from Orange.widgets.widget import Input, Output, Msg
 
-INSTANCEID = "Instance id"
-INDEX = "Row index"
+INSTANCEID = "实例 ID"
+INDEX = "行索引"
 
 
 class ConditionBox(QWidget):
@@ -69,7 +73,7 @@ class ConditionBox(QWidget):
             self.emit_list()
 
         def get_combo(model):
-            combo = QComboBox(self)
+            combo = ComboBoxSearch(self)
             combo.setModel(model)
             # We use signal activated because it is triggered only on user
             # interaction, not programmatically.
@@ -85,7 +89,7 @@ class ConditionBox(QWidget):
 
         row = self.layout().count()
         row_items = self.RowItems(
-            QLabel("and" if row else self.pre_label),
+            QLabel("和" if row else self.pre_label),
             get_combo(self.model_left),
             QLabel(self.in_label),
             get_combo(self.model_right),
@@ -217,8 +221,13 @@ class MergeDataContextHandler(ContextHandler):
         context = widget.current_context
         if context is None:
             return
-        pairs = context.values.get("attr_pairs", [])
-        widget.attr_pairs = [self.decode_pair(widget, pair) for pair in pairs]
+        pairs = context.values.get("attr_pairs")
+        if pairs:
+            # attr_pairs is schema only setting which means it is not always
+            # present. When not present leave widgets default.
+            widget.attr_pairs = [
+                self.decode_pair(widget, pair) for pair in pairs
+            ]
 
     def match(self, context, variables1, variables2):
         def matches(part, variables):
@@ -228,9 +237,10 @@ class MergeDataContextHandler(ContextHandler):
         if (variables1, variables2) == (context.variables1, context.variables2):
             return self.PERFECT_MATCH
 
-        left, right = zip(*context.values["attr_pairs"])
-        if matches(left, variables1) and matches(right, variables2):
-            return 0.5
+        if "attr_pairs" in context.values:
+            left, right = zip(*context.values["attr_pairs"])
+            if matches(left, variables1) and matches(right, variables2):
+                return 0.5
 
         return self.NO_MATCH
 
@@ -240,21 +250,22 @@ class OWMergeData(widget.OWWidget):
     description = "根据选择的特征合并数据集."
     icon = "icons/MergeData.svg"
     priority = 1110
-    keywords = ["join"]
+    keywords = ["join", 'hebing']
+    category = "Data"
 
     class Inputs:
-        data = Input(" 数据(Data)", Orange.data.Table, default=True, replaces=["Data A", 'Data'])
+        data = Input("主数据(Data)", Orange.data.Table, default=True, replaces=["Data A", 'Data'])
         extra_data = Input("附加数据(Extra Data)", Orange.data.Table, replaces=["Data B", 'Extra Data'])
 
     class Outputs:
         data = Output("数据(Data)",
                       Orange.data.Table,
-                      replaces=["Merged Data A+B", "Merged Data B+A", "Merged Data", 'Data'])
+                      replaces=["Data", "Merged Data A+B", "Merged Data B+A", "Merged Data"])
 
     LeftJoin, InnerJoin, OuterJoin = range(3)
-    OptionNames = ("从附加数据添加列(Left Join)",
-                   "寻找匹配行(Inner Join)",
-                   "连接表格 (Outer Join)")
+    OptionNames = ("从附加数据添加列(左连接)",
+                   "寻找匹配行(内连接)",
+                   "连接表格 (全外连接)")
     OptionDescriptions = (
         "附加数据中的列将添加到数据中. 没有匹配行的实例将添加缺失值。",
 
@@ -304,10 +315,8 @@ class OWMergeData(widget.OWWidget):
         self.model = DomainModelWithTooltips(content)
         self.extra_model = DomainModelWithTooltips(content)
 
-        box = gui.hBox(self.controlArea, box=None)
-        no_info = self.data_info(None)
-        self.info_box_data = gui.label(box, self, no_info, box="数据(Data)")
-        self.info_box_extra_data = gui.label(box, self, no_info, box="附加数据(Extra Data)")
+        self.info.set_input_summary(self.info.NoInput)
+        self.info.set_output_summary(self.info.NoOutput)
 
         grp = gui.radioButtons(
             self.controlArea, self, "merging", box="合并",
@@ -335,14 +344,12 @@ class OWMergeData(widget.OWWidget):
     def set_data(self, data):
         self.data = data
         self.model.set_domain(data and data.domain)
-        self.info_box_data.setText(self.data_info(data))
 
     @Inputs.extra_data
     @check_sql_input
     def set_extra_data(self, data):
         self.extra_data = data
         self.extra_model.set_domain(data and data.domain)
-        self.info_box_extra_data.setText(self.data_info(data))
 
     def store_combo_state(self):
         self.attr_pairs = self.attr_boxes.current_state()
@@ -353,6 +360,20 @@ class OWMergeData(widget.OWWidget):
         self.openContext(self.data and self.data.domain,
                          self.extra_data and self.extra_data.domain)
         self.attr_boxes.set_state(self.attr_pairs)
+
+        summary, details, kwargs = self.info.NoInput, "", {}
+        if self.data or self.extra_data:
+            n_data = len(self.data) if self.data else 0
+            n_extra_data = len(self.extra_data) if self.extra_data else 0
+            summary = f"{self.info.format_number(n_data)}, " \
+                      f"{self.info.format_number(n_extra_data)}"
+            kwargs = {"format": Qt.RichText}
+            details = format_multiple_summaries([
+                ("Data", self.data),
+                ("Extra data", self.extra_data)
+            ])
+        self.info.set_input_summary(summary, details, **kwargs)
+
         self.unconditional_commit()
 
     def _find_best_match(self):
@@ -373,20 +394,13 @@ class OWMergeData(widget.OWWidget):
                     n_max_intersect, attr, extra_attr = n_inter, m_a, m_b
         return attr, extra_attr
 
-    @staticmethod
-    def data_info(data):
-        if data is None:
-            return "无数据."
-        else:
-            return \
-                f"{data.name}\n" \
-                f"{len(data)} 个实例\n" \
-                f"{len(data.domain) + len(data.domain.metas)} 个变量"
-
     def commit(self):
         self.clear_messages()
         merged = self.merge() if self.data and self.extra_data else None
         self.Outputs.data.send(merged)
+        details = format_summary_details(merged) if merged else ""
+        summary = len(merged) if merged else self.info.NoOutput
+        self.info.set_output_summary(summary, details)
 
     def send_report(self):
         # pylint: disable=invalid-sequence-index
@@ -502,7 +516,7 @@ class OWMergeData(widget.OWWidget):
             if var.is_discrete:
                 col = col.astype(int)
                 col[nans] = len(var.values)
-                col = np.array(var.values + [np.nan])[col]
+                col = np.array(var.values + (np.nan, ))[col]
         else:
             col = col.copy()
             defined = col.astype(bool)
