@@ -1,7 +1,7 @@
 # pylint: disable=missing-docstring
 from typing import Iterable, Set
 from collections import defaultdict
-from itertools import product
+from itertools import product, chain
 
 import numpy as np
 from scipy import sparse as sp
@@ -10,11 +10,13 @@ from AnyQt.QtCore import (Qt, QSize, QItemSelection, QItemSelectionModel,
                           pyqtSignal)
 from AnyQt.QtGui import QStandardItem, QColor, QStandardItemModel
 from AnyQt.QtWidgets import (QTableView, QSizePolicy, QHeaderView,
-                             QStyledItemDelegate, QCheckBox, QFrame)
+                             QStyledItemDelegate, QCheckBox, QFrame, QWidget,
+                             QGridLayout)
 
 from Orange.data import (Table, DiscreteVariable, Variable, Domain,
-                         ContinuousVariable)
+                         ContinuousVariable, TimeVariable)
 from Orange.data.domain import filter_visible
+from Orange.data.util import get_unique_names_duplicates, get_unique_names
 from Orange.data.filter import FilterContinuous, FilterDiscrete, Values
 from Orange.statistics.util import (nanmin, nanmax, nanunique, nansum, nanvar,
                                     nanmean, nanmedian, nanmode, bincount)
@@ -25,6 +27,8 @@ from Orange.widgets.settings import (Setting, ContextSetting,
 from Orange.widgets.utils.sql import check_sql_input
 from Orange.widgets.utils.itemmodels import DomainModel
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details, \
+    format_multiple_summaries
 from Orange.widgets.widget import OWWidget, Input, Output, Msg
 
 
@@ -36,8 +40,22 @@ class AggregationFunctionsEnum(Enum):
     (Count, Count_defined, Sum, Mean, Min, Max,
      Mode, Median, Var, Majority) = range(10)
 
+
+
     def __init__(self, *_, **__):
         super().__init__()
+        self.chinese_dict = {
+            'Count': '数目',
+            'Count_defined': '非缺失数目',
+            'Sum': '总和',
+            'Mean': '平均值',
+            'Min': '最小值',
+            'Max': '最大值',
+            'Mode': '众数',
+            'Median': '中位数',
+            'Var': '方差',
+            'Majority': '最常见',
+        }       
         self.func = None
 
     @property
@@ -48,7 +66,8 @@ class AggregationFunctionsEnum(Enum):
         return self.func(args)  # pylint: disable=not-callable
 
     def __str__(self):
-        return self._name_.replace("_", " ")
+        # return self._name_.replace("_", " ")
+        return self.chinese_dict[self._name_]
 
     def __gt__(self, other):
         return self._value_ > other.value
@@ -63,6 +82,8 @@ class Pivot:
     AnyVarFunctions = (Count_defined,)
     ContVarFunctions = (Sum, Mean, Min, Max, Mode, Median, Var)
     DiscVarFunctions = (Majority,)
+    TimeVarFunctions = (Mean, Min, Max, Mode, Median)
+    FloatFunctions = (Count, Count_defined, Sum, Var)
 
     class Tables:
         table = None  # type: Table
@@ -81,6 +102,7 @@ class Pivot:
         self._table = table
         self._row_var = row_var
         self._col_var = col_var if col_var else row_var
+        self.renamed = []
 
         if not table:
             return
@@ -95,7 +117,7 @@ class Pivot:
         self._row_var_groups = nanunique(self._row_var_col)
         self._col_var_groups = nanunique(self._col_var_col)
 
-        self._total_var = DiscreteVariable("Total", values=["total"])
+        self._total_var = DiscreteVariable("Total", values=("total", ))
         self._current_agg_functions = sorted(agg_funs)
         self._indepen_agg_done = {}  # type: Dict[Functions, int]
         self._depen_agg_done = {}  # type: Dict[Functions, Dict[Variable, int]]
@@ -190,7 +212,12 @@ class Pivot:
             if fun in self.DiscVarFunctions:
                 attrs.append(DiscreteVariable(name, var.values))
             else:
-                attrs.append(ContinuousVariable(name))
+                if isinstance(var, TimeVariable) and \
+                        fun in self.TimeVarFunctions:
+                    attrs.append(TimeVariable(name, have_date=var.have_date,
+                                              have_time=var.have_time))
+                else:
+                    attrs.append(ContinuousVariable(name))
         args = (var_indep_funs, var_dep_funs, attrs)
         for t, var in (("table", None), ("total_h", self._col_var),
                        ("total_v", self._row_var), ("total", self._total_var)):
@@ -251,7 +278,19 @@ class Pivot:
                     X[:, k] = group_tab.X[:, self._depen_agg_done[fun][v] - offset]
                 else:
                     X[i, k] = fun(sub_table.get_column_view(v)[0])
-        return Table(Domain(leading_vars + attrs), np.hstack((combs, X)))
+
+        #rename leading vars (seems the easiest) if needed
+        current = [var.name for var in attrs]
+        uniq_leading_vars = []
+        for v in leading_vars:
+            uniq = get_unique_names(current, v.name)
+            if uniq != v.name:
+                self.renamed.append(v.name)
+                v = v.copy(name=uniq)
+            uniq_leading_vars.append(v)
+            current.append(uniq)
+
+        return Table(Domain(uniq_leading_vars + attrs), np.hstack((combs, X)))
 
     def update_pivot_table(self, val_var: Variable):
         self._create_pivot_tables(val_var)
@@ -280,11 +319,23 @@ class Pivot:
                 _X[:, index][_X[:, index] == value] = j
             return values
 
+        create_time_var = \
+            isinstance(val_var, TimeVariable) and \
+            all(fun in self.TimeVarFunctions for fun in agg_funs)
+        create_cont_var = \
+            not val_var or val_var.is_continuous and \
+            (not isinstance(val_var, TimeVariable) or
+             all(fun in self.FloatFunctions for fun in agg_funs))
+
         vals = np.array(self._col_var.values)[self._col_var_groups.astype(int)]
-        if not val_var or val_var.is_continuous:
-            cv = ContinuousVariable
-            attrs = [[cv(f"{v}", 1) for v in vals]] * 2
-            attrs.extend([[cv("Total", 1)]] * 2)
+        if create_time_var:
+            kwargs = {"have_date": val_var.have_date,
+                      "have_time": val_var.have_time}
+            attrs = [[TimeVariable(f"{v}", **kwargs) for v in vals]] * 2
+            attrs.extend([[TimeVariable("Total", **kwargs)]] * 2)
+        elif create_cont_var:
+            attrs = [[ContinuousVariable(f"{v}", 1) for v in vals]] * 2
+            attrs.extend([[ContinuousVariable("Total", 1)]] * 2)
         else:
             attrs = []
             for x in (X, X_h):
@@ -293,7 +344,36 @@ class Pivot:
             for x in (X_v, X_t):
                 attrs.append([DiscreteVariable("Total", map_values(0, x))])
         row_var_h = DiscreteVariable(self._row_var.name, values=["Total"])
-        aggr_attr = DiscreteVariable("Aggregate", [str(f) for f in agg_funs])
+        aggr_attr = DiscreteVariable('Aggregate', [str(f) for f in agg_funs])
+
+        same_row_col = self._col_var is self._row_var
+
+        extra_vars = [self._row_var, aggr_attr]
+        uniq_a = get_unique_names_duplicates([v.name for v in extra_vars]
+                                             + [atr.name for atr in attrs[0]])
+        for (idx, var), u in zip(enumerate(chain(extra_vars, attrs[0])), uniq_a):
+            if var.name == u:
+                continue
+            if idx == 0:
+                self.renamed.append(self._row_var.name)
+                self._row_var = self._row_var.copy(name=u)
+                if same_row_col:
+                    self._col_var = self._row_var
+                row_var_h = row_var_h.copy(name=u)
+            elif idx == 1:
+                self.renamed.append(aggr_attr.name)
+                aggr_attr = aggr_attr.copy(name=u)
+            else:
+                self.renamed.append(var.name)
+                attrs[0][idx-2] = var.copy(name=u)
+                attrs[1][idx-2] = var.copy(name=u)
+
+        if same_row_col:
+            vals = tuple(v.name for v in attrs[0])
+            self._row_var.make(self._row_var.name, values=vals)
+            vals = tuple(v.name for v in attrs[2])
+            row_var_h.make(row_var_h.name, vals)
+
         return (Domain([self._row_var, aggr_attr] + attrs[0]),
                 Domain([row_var_h, aggr_attr] + attrs[1]),
                 Domain(attrs[2]), Domain(attrs[3]))
@@ -302,15 +382,19 @@ class Pivot:
         gt = self._group_tables
         n_fun = len(agg_funs)
         n_rows, n_cols = len(self._row_var_groups), len(self._col_var_groups)
-        kwargs = {"fill_value": np.nan, "dtype": float} \
-            if not val_var or val_var.is_continuous \
+        is_float_type = not val_var or val_var.is_continuous
+        if isinstance(val_var, TimeVariable):
+            is_float_type = \
+                all(fun in self.TimeVarFunctions for fun in agg_funs) or \
+                all(fun in self.FloatFunctions for fun in agg_funs)
+        kwargs = {"fill_value": np.nan, "dtype": float} if is_float_type \
             else {"fill_value": "", "dtype": object}
         X = np.full((n_rows * n_fun, 2 + n_cols), **kwargs)
         X_h = np.full((n_fun, 2 + n_cols), **kwargs)
         X_v = np.full((n_rows * n_fun, 1), **kwargs)
         X_t = np.full((n_fun, 1), **kwargs)
         for i, fun in enumerate(agg_funs):
-            args = (val_var, fun)
+            args = (val_var, fun, is_float_type)
             X[i::n_fun, 2:] = self.__rows_for_function(n_rows, n_cols, *args)
             X[i::n_fun, :2] = np.array([[row_val, agg_funs.index(fun)] for
                                         row_val in self._row_var_groups])
@@ -320,13 +404,14 @@ class Pivot:
             X_t[i] = self.__total_for_function(gt.total, *args)
         return X, X_h, X_v, X_t
 
-    def __total_for_function(self, group_tab, val_var, fun):
+    def __total_for_function(self, group_tab, val_var, fun, is_float_type):
         ref = self._indepen_agg_done.get(fun, None) \
               or self._depen_agg_done[fun][val_var]
         ref -= int(bool(not self.single_var_grouping))
-        return self.__check_continuous(val_var, group_tab.X[:, ref], fun)
+        return self.__check_continuous(val_var, group_tab.X[:, ref],
+                                       fun, is_float_type)
 
-    def __rows_for_function(self, n_rows, n_cols, val_var, fun):
+    def __rows_for_function(self, n_rows, n_cols, val_var, fun, is_float_type):
         ref = self._indepen_agg_done.get(fun, None) \
               or self._depen_agg_done[fun][val_var]
         column = self._group_tables.table.X[:, ref]
@@ -335,14 +420,23 @@ class Pivot:
             rows[np.diag_indices_from(rows)] = column
         else:
             rows = column.reshape(n_rows, n_cols)
-        return self.__check_continuous(val_var, rows, fun)
+        return self.__check_continuous(val_var, rows, fun, is_float_type)
 
-    def __check_continuous(self, val_var, column, fun):
+    def __check_continuous(self, val_var, column, fun, is_float_type):
         if val_var and not val_var.is_continuous:
             column = column.astype(str)
             if fun in self.DiscVarFunctions:
                 for j, val in enumerate(val_var.values):
                     column[column == str(float(j))] = val
+        elif isinstance(val_var, TimeVariable) and not is_float_type:
+            shape = column.shape
+            column = column.flatten()
+            column_ = column.astype(str)
+            if fun in self.TimeVarFunctions:
+                for i in range(column.shape[0]):
+                    if not np.isnan(column[i]):
+                        column_[i] = val_var.repr_val(column[i])
+            return column_.reshape(shape)
         return column
 
     @staticmethod
@@ -567,7 +661,8 @@ class PivotTableView(QTableView):
 
     def _set_values(self, table):
         for i, j in product(range(len(table)), range(len(table[0]))):
-            item = self._create_value_item(str(table[i, j]))
+            value = table[i, j]
+            item = self._create_value_item(str(value))
             self.table_model.setItem(i + self._n_leading_rows,
                                      j + self._n_leading_cols, item)
 
@@ -649,7 +744,8 @@ class OWPivot(OWWidget):
     description = "根据列值重新调整数据表的形状。"
     icon = "icons/Pivot.svg"
     priority = 1000
-    keywords = ["pivot", "group", "aggregate"]
+    keywords = ["pivot", "group", "aggregate", 'toushibiao']
+    category = "Data"
 
     class Inputs:
         data = Input("数据(Data)", Table, default=True, replaces=['Data'])
@@ -662,61 +758,82 @@ class OWPivot(OWWidget):
     class Warning(OWWidget.Warning):
         # TODO - inconsistent for different variable types
         no_col_feature = Msg("Column feature should be selected.")
-        cannot_aggregate = Msg("({}) 无法执行.")
+        cannot_aggregate = Msg("某些聚合 ({}) 无法执行.")
+        renamed_vars = Msg("为防止重复, 某些变量已经重命名\n{}")
+        too_many_values = Msg("选中的变量有过多的值.")
 
     settingsHandler = DomainContextHandler()
     row_feature = ContextSetting(None)
     col_feature = ContextSetting(None)
     val_feature = ContextSetting(None)
     sel_agg_functions = Setting(set([Pivot.Count]))
-    selection = ContextSetting(set())
+    selection = Setting(set(), schema_only=True)
     auto_commit = Setting(True)
 
     AGGREGATIONS = (Pivot.Count,
                     Pivot.Count_defined,
-                    None,
+                    None,  # separator
                     Pivot.Sum,
                     Pivot.Mean,
+                    Pivot.Var,
+                    Pivot.Median,
+                    2,  # column break
                     Pivot.Mode,
                     Pivot.Min,
                     Pivot.Max,
-                    Pivot.Median,
-                    Pivot.Var,
                     None,
                     Pivot.Majority)
 
+    MAX_VALUES = 100
+
     def __init__(self):
         super().__init__()
         self.data = None  # type: Table
         self.pivot = None  # type: Pivot
+        self.__pending_selection = self.selection  # type: Set
         self._add_control_area_controls()
         self._add_main_area_controls()
 
     def _add_control_area_controls(self):
-        box = gui.vBox(self.controlArea, "行")
-        gui.comboBox(box, self, "row_feature", contentsLength=12,
+        box = gui.vBox(self.controlArea, box=True)
+        gui.comboBox(box, self, "row_feature", label="行", contentsLength=12,
+                     searchable=True,
                      model=DomainModel(valid_types=DomainModel.PRIMITIVE),
                      callback=self.__feature_changed)
-        box = gui.vBox(self.controlArea, "列")
-        gui.comboBox(box, self, "col_feature", contentsLength=12,
+        gui.comboBox(box, self, "col_feature", label="列",
+                     contentsLength=12,
+                     searchable=True,
                      model=DomainModel(placeholder="(与行相同)",
                                        valid_types=DiscreteVariable),
-                     callback=self.__feature_changed)
-        box = gui.vBox(self.controlArea, "值")
-        gui.comboBox(box, self, "val_feature", contentsLength=12,
-                     model=DomainModel(placeholder="(None)"),
-                     orientation=Qt.Horizontal,
+                     callback=self.__feature_changed,)
+        gui.comboBox(box, self, "val_feature", label="值",
+                     contentsLength=12,
+                     searchable=True,
+                     model=DomainModel(placeholder="(无)"),
                      callback=self.__val_feature_changed)
         self.__add_aggregation_controls()
         gui.rubber(self.controlArea)
         gui.auto_apply(self.controlArea, self, "auto_commit")
 
+        self.set_input_summary()
+        self.set_output_summary(None, None, None)
+
     def __add_aggregation_controls(self):
+        def new_inbox():
+            nonlocal row, col, inbox
+            inbox = QWidget()
+            layout = QGridLayout()
+            inbox.setLayout(layout)
+            layout.setContentsMargins(0, 0, 0, 0)
+            box.layout().addWidget(inbox)
+            row = col = 0
+
         box = gui.vBox(self.controlArea, "聚合")
-        chinese_aggs=["计数(Count)","计数已定义项(Count_defined)",
-        None,"总和(Sum)","平均(Mean)","样式(Mode)","最小(Min)",
-        "最大(Max)","中位数(Median)","变量(Var)",None,"大多数(Majority)"]
-        for agg,chinese_agg in zip(self.AGGREGATIONS,chinese_aggs):
+        row = col = 0
+        inbox = None
+        new_inbox()
+        self.aggregation_checkboxes = []  # for test purposes
+        for agg in self.AGGREGATIONS:
             if agg is None:
                 gui.separator(box, height=1)
                 line = QFrame()
@@ -724,12 +841,19 @@ class OWPivot(OWWidget):
                 line.setLineWidth(1)
                 line.setFrameShadow(QFrame.Sunken)
                 box.layout().addWidget(line)
+                new_inbox()
                 continue
-            check_box = QCheckBox(str(chinese_agg), box)
+            elif agg == 2:
+                col += 1
+                row = 0
+                continue
+            check_box = QCheckBox(str(agg), inbox)
             check_box.setChecked(agg in self.sel_agg_functions)
             check_box.clicked.connect(lambda *args, a=agg:
                                       self.__aggregation_cb_clicked(a, args[0]))
-            box.layout().addWidget(check_box)
+            inbox.layout().addWidget(check_box, row, col)
+            self.aggregation_checkboxes.append(check_box)
+            row += 1
 
     def _add_main_area_controls(self):
         self.table_view = PivotTableView()
@@ -782,6 +906,7 @@ class OWPivot(OWWidget):
     @check_sql_input
     def set_data(self, data):
         self.closeContext()
+        self.selection = set()
         self.data = data
         self.pivot = None
         self.check_data()
@@ -791,8 +916,7 @@ class OWPivot(OWWidget):
 
     def check_data(self):
         self.clear_messages()
-        if not self.data:
-            self.table_view.clear()
+        self.set_input_summary()
 
     def init_attr_values(self):
         domain = self.data.domain if self.data and len(self.data) else None
@@ -808,21 +932,69 @@ class OWPivot(OWWidget):
                 if domain.variables[0] in model else model[2]
 
     def commit(self):
+        def send_outputs(pivot_table, filtered_data, grouped_data):
+            self.Outputs.grouped_data.send(grouped_data)
+            self.Outputs.pivot_table.send(pivot_table)
+            self.Outputs.filtered_data.send(filtered_data)
+            self.set_output_summary(pivot_table, filtered_data, grouped_data)
+
+        self.Warning.renamed_vars.clear()
+        self.Warning.too_many_values.clear()
+        self.Warning.cannot_aggregate.clear()
+        self.Warning.no_col_feature.clear()
+
         if self.pivot is None:
-            self.Warning.no_col_feature.clear()
             if self.no_col_feature:
+                self.table_view.clear()
                 self.Warning.no_col_feature()
+                send_outputs(None, None, None)
                 return
+
+            if self.data:
+                col_var = self.col_feature or self.row_feature
+                col = self.data.get_column_view(col_var)[0].astype(np.float)
+                if len(nanunique(col)) >= self.MAX_VALUES:
+                    self.table_view.clear()
+                    self.Warning.too_many_values()
+                    send_outputs(None, None, None)
+                    return
+
             self.pivot = Pivot(self.data, self.sel_agg_functions,
                                self.row_feature,
                                self.col_feature, self.val_feature)
-        self.Warning.cannot_aggregate.clear()
+
         if self.skipped_aggs:
             self.Warning.cannot_aggregate(self.skipped_aggs)
         self._update_graph()
-        self.Outputs.grouped_data.send(self.pivot.group_table)
-        self.Outputs.pivot_table.send(self.pivot.pivot_table)
-        self.Outputs.filtered_data.send(self.get_filtered_data())
+
+        send_outputs(self.pivot.pivot_table,
+                     self.get_filtered_data(),
+                     self.pivot.group_table)
+
+        if self.pivot.renamed:
+            self.Warning.renamed_vars(self.pivot.renamed)
+
+    def set_input_summary(self):
+        summary = len(self.data) if self.data else self.info.NoInput
+        details = format_summary_details(self.data) if self.data else ""
+        self.info.set_input_summary(summary, details)
+
+    def set_output_summary(self, pivot: Table, filtered: Table, grouped: Table):
+        summary, detail, kwargs = self.info.NoOutput, "", {}
+        if pivot or filtered or grouped:
+            n_pivot = len(pivot) if pivot else 0
+            n_filtered = len(filtered) if filtered else 0
+            n_grouped = len(grouped) if grouped else 0
+            summary = f"{self.info.format_number(n_pivot)}, " \
+                      f"{self.info.format_number(n_filtered)}, " \
+                      f"{self.info.format_number(n_grouped)}"
+            detail = format_multiple_summaries([
+                ("Pivot table", pivot),
+                ("Filtered data", filtered),
+                ("Grouped data", grouped)
+            ])
+            kwargs = {"format": Qt.RichText}
+        self.info.set_output_summary(summary, detail, **kwargs)
 
     def _update_graph(self):
         self.table_view.clear()
@@ -831,7 +1003,10 @@ class OWPivot(OWWidget):
             self.table_view.update_table(col_feature.name,
                                          self.row_feature.name,
                                          *self.pivot.pivot_tables)
-            self.table_view.set_selection(self.selection)
+            selection = self.__pending_selection or self.selection
+            self.table_view.set_selection(selection)
+            self.selection = self.table_view.get_selection()
+            self.__pending_selection = set()
 
     def get_filtered_data(self):
         if not self.data or not self.selection or not self.pivot.pivot_table:
@@ -847,9 +1022,10 @@ class OWPivot(OWWidget):
                 elif isinstance(at, ContinuousVariable):
                     f.append(FilterContinuous(at, FilterContinuous.Equal, val))
             cond.append(Values(f))
-        return Values([f for f in cond], conjunction=False)(self.data)
+        return Values(cond, conjunction=False)(self.data)
 
-    def sizeHint(self):
+    @staticmethod
+    def sizeHint():
         return QSize(640, 525)
 
     def send_report(self):
