@@ -1,9 +1,10 @@
 # Test methods with long descriptive names can omit docstrings
 # pylint: disable=missing-docstring
 # There are never too many tests, so:
-# pylint: disable=too-many-lines,too-many-public-methods
+# pylint: disable=too-many-lines,too-many-public-methods, protected-access
 from itertools import chain
 import unittest
+from types import SimpleNamespace
 from unittest.mock import Mock
 
 import numpy as np
@@ -16,6 +17,8 @@ from Orange.data import Table, Domain, DiscreteVariable, StringVariable, \
 from Orange.widgets.data.owmergedata import OWMergeData, INSTANCEID, INDEX, \
     MergeDataContextHandler
 from Orange.widgets.tests.base import WidgetTest
+from Orange.widgets.utils.state_summary import format_multiple_summaries, \
+    format_summary_details
 from Orange.tests import test_filename
 
 
@@ -779,15 +782,15 @@ class TestOWMergeData(WidgetTest):
         data = Table("iris")[::25]
         data_ed_dense = Table("titanic")[::300]
         data_ed_sparse = Table("titanic")[::300].to_sparse()
-        self.send_signal("Data", data)
+        self.send_signal("主数据(Data)", data)
 
-        self.send_signal("Extra Data", data_ed_dense)
-        output_dense = self.get_output("Data")
+        self.send_signal("附加数据(Extra Data)", data_ed_dense)
+        output_dense = self.get_output("数据(Data)")
         self.assertFalse(sp.issparse(output_dense.X))
         self.assertFalse(output_dense.is_sparse())
 
-        self.send_signal("Extra Data", data_ed_sparse)
-        output_sparse = self.get_output("Data")
+        self.send_signal("附加数据(Extra Data)", data_ed_sparse)
+        output_sparse = self.get_output("数据(Data)")
         self.assertTrue(sp.issparse(output_sparse.X))
         self.assertTrue(output_sparse.is_sparse())
 
@@ -833,7 +836,7 @@ class TestOWMergeData(WidgetTest):
     def test_nonunique(self):
         widget = self.widget
         x = ContinuousVariable("x")
-        d = DiscreteVariable("d", values=list("abc"))
+        d = DiscreteVariable("d", values=tuple("abc"))
         domain = Domain([x, d], [])
         dataA = Table.from_numpy(
             domain, np.array([[1.0, 0], [1, 1], [2, 1]]))
@@ -905,7 +908,7 @@ class TestOWMergeData(WidgetTest):
     def test_invalide_pairs(self):
         widget = self.widget
         x = ContinuousVariable("x")
-        d = DiscreteVariable("d", values=list("abc"))
+        d = DiscreteVariable("d", values=tuple("abc"))
         domain = Domain([x, d], [])
         dataA = Table.from_numpy(
             domain, np.array([[1.0, 0], [1, 1], [2, 1]]))
@@ -959,7 +962,7 @@ class TestOWMergeData(WidgetTest):
 
     def test_duplicate_names(self):
         domain = Domain([ContinuousVariable("C1")],
-                        metas=[DiscreteVariable("Feature", values=["A", "B"])])
+                        metas=[DiscreteVariable("Feature", values=("A", "B"))])
         data = Table(domain, np.array([[1.], [0.]]),
                      metas=np.array([[1.], [0.]]))
         domain = Domain([ContinuousVariable("C1")],
@@ -984,7 +987,7 @@ class TestOWMergeData(WidgetTest):
                              ["A", "B (1)", "B (2)"])
 
     def test_keep_non_duplicate_variables_missing_rows(self):
-        c = DiscreteVariable("C", values=["a", "b", "c"])
+        c = DiscreteVariable("C", values=("a", "b", "c"))
         domain = Domain([ContinuousVariable("A"), ContinuousVariable("B"), c])
         data = Table(domain, np.array([[0., 0, 0], [1, 1, 1]]))
         extra_data = Table(domain, np.array([[0., 1, 1], [0, 1, 2]]))
@@ -1019,6 +1022,44 @@ class TestOWMergeData(WidgetTest):
         self.assertListEqual([m.name for m in merged_data.domain.variables],
                              ["A", "B", "C"])
 
+    def test_summary(self):
+        """Check if the status bar is updated when data is received"""
+        data = self.dataA
+        info = self.widget.info
+        no_input, no_output = "No data on input", "No data on output"
+
+        self.send_signal(self.widget.Inputs.data, data)
+        data_list = [("Data", data), ("Extra data", None)]
+        summary, details = f"{len(data)}, 0", format_multiple_summaries(data_list)
+        self.assertEqual(info._StateInfo__input_summary.brief, summary)
+        self.assertEqual(info._StateInfo__input_summary.details, details)
+        self.assertEqual(info._StateInfo__output_summary.brief, "-")
+        self.assertEqual(info._StateInfo__output_summary.details, no_output)
+
+        self.send_signal(self.widget.Inputs.extra_data, data)
+        data_list = [("Data", data), ("Extra data", data)]
+        summary = f"{len(data)}, {len(data)}"
+        details = format_multiple_summaries(data_list)
+        self.assertEqual(info._StateInfo__input_summary.brief, summary)
+        self.assertEqual(info._StateInfo__input_summary.details, details)
+        output = self.get_output(self.widget.Outputs.data)
+        summary, details = f"{len(output)}", format_summary_details(output)
+        self.assertEqual(info._StateInfo__output_summary.brief, summary)
+        self.assertEqual(info._StateInfo__output_summary.details, details)
+
+        self.send_signal(self.widget.Inputs.data, None)
+        data_list = [("Data", None), ("Extra data", data)]
+        summary, details = f"0, {len(data)}", format_multiple_summaries(data_list)
+        self.assertEqual(info._StateInfo__input_summary.brief, summary)
+        self.assertEqual(info._StateInfo__input_summary.details, details)
+        self.assertEqual(info._StateInfo__output_summary.brief, "-")
+        self.assertEqual(info._StateInfo__output_summary.details, no_output)
+
+        self.send_signal(self.widget.Inputs.extra_data, None)
+        self.assertEqual(info._StateInfo__input_summary.brief, "-")
+        self.assertEqual(info._StateInfo__input_summary.details, no_input)
+        self.assertEqual(info._StateInfo__output_summary.brief, "-")
+        self.assertEqual(info._StateInfo__output_summary.details, no_output)
 
 class MergeDataContextHandlerTest(unittest.TestCase):
     # These units are too small to test individually, so they are tested
@@ -1036,6 +1077,37 @@ class MergeDataContextHandlerTest(unittest.TestCase):
         handler.settings_from_widget(widget)  # mustn't crash
         handler.settings_to_widget(widget)  # mustn't crash
 
+    def test_attr_pairs_not_present(self):
+        data = Table("iris")
+
+        context = SimpleNamespace(values={})
+        widget = SimpleNamespace(
+            current_context=context, attr_pairs=("a", "b")
+        )
+        handler = MergeDataContextHandler()
+
+        handler.settings_to_widget(widget)  # mustn't crash
+        # no attr_pairs in context -> handler must not change widget.attr_pairs
+        self.assertTupleEqual(widget.attr_pairs, ("a", "b"))
+
+        context = SimpleNamespace(
+            values={
+                "attr_pairs": [((data.domain[0], 100), (data.domain[1], 100))]
+            }
+        )
+        widget = SimpleNamespace(
+            current_context=context,
+            attr_pairs=("a", "b"),
+            data=data,
+            extra_data=data,
+        )
+
+        handler.settings_to_widget(widget)  # mustn't crash
+        # values taken from context
+        self.assertListEqual(
+            widget.attr_pairs, [(data.domain[0], data.domain[1])]
+        )
+
 
 if __name__ == "__main__":
     unittest.main()
