@@ -2,12 +2,15 @@ from Orange.widgets.widget import OWWidget
 from Orange.widgets import gui
 from pathlib import Path
 from PyQt5.QtCore import QProcess
+import sys
 
 
 class Crawler(OWWidget):
     name = "爬行者(Crawler)"
     description = "训练一个爬行机器人, 熟悉强化学习的各种参数"
     icon = "icons/robot.svg"
+    keywords = ['paixingzhe']
+    category = 'reinforcement'
 
     want_main_area = False
 
@@ -19,7 +22,10 @@ class Crawler(OWWidget):
     def commit(self):
         dir_path = Path(__file__).resolve()
         parent_path = dir_path.parent.parent
-        command = f'{str(parent_path)}/binaries/crawler.exe'
+        if sys.platform.startswith('win'):
+            command = f'{str(parent_path)}/binaries/crawler.exe'
+        else:
+            command = f'{str(parent_path)}/binaries/crawler'
 
         self.process = QProcess(self)
 
