@@ -7,6 +7,7 @@ from unittest.mock import Mock, patch
 import numpy as np
 import scipy.sparse as sp
 
+from AnyQt.QtGui import QFont, QTextDocumentFragment
 from AnyQt.QtCore import QRectF, QPointF
 from AnyQt.QtTest import QSignalSpy
 from AnyQt.QtWidgets import (
@@ -31,13 +32,18 @@ from Orange.regression.base_regression import (
 )
 from Orange.widgets.tests.utils import simulate
 from Orange.widgets.utils.annotated_data import (
-    ANNOTATED_DATA_FEATURE_NAME, ANNOTATED_DATA_SIGNAL_NAME
+    ANNOTATED_DATA_FEATURE_NAME, ANNOTATED_DATA_SIGNAL_Chinese_NAME
 )
 from Orange.widgets.utils.owlearnerwidget import OWBaseLearner
+from Orange.widgets.visualize.utils.plotutils import AnchorItem
 from Orange.widgets.widget import OWWidget
 
 
 class WidgetTest(WidgetTestBase):
+    '''
+    Assert two tables are equal
+    '''
+
     def assert_table_equal(self, table1, table2):
         if table1 is None or table2 is None:
             self.assertIs(table1, table2)
@@ -183,7 +189,7 @@ class ParameterMapping(BaseParameterMapping):
     @staticmethod
     def _default_get_value(gui_element, values):
         if isinstance(gui_element, (QSpinBox, QDoubleSpinBox, QSlider)):
-            return lambda: gui_element.value()
+            return gui_element.value
         elif isinstance(gui_element, QComboBox):
             return lambda: values[gui_element.currentIndex()]
         else:
@@ -218,9 +224,11 @@ class WidgetLearnerTestMixin:
     widget = None  # type: OWBaseLearner
 
     def init(self):
+        # init dataset for classification and regression
         cls_ds = Table(datasets.path("testing_dataset_cls"))
         reg_ds = Table(datasets.path("testing_dataset_reg"))
 
+        # init models
         if issubclass(self.widget.LEARNER, Fitter):
             self.data = cls_ds
             self.valid_datasets = (cls_ds, reg_ds)
@@ -246,22 +254,27 @@ class WidgetLearnerTestMixin:
         self.parameters = []
 
     def test_has_unconditional_apply(self):
+        '''
+        If there is a new signal comes in, handle that signal immediately
+        widgets should Apply, Send or Run when their input signals changed even if auto-apply/send/run is disabled.
+        see: https://github.com/biolab/orange3/issues/3455#issuecomment-449428361
+        '''
         self.assertTrue(hasattr(self.widget, "unconditional_apply"))
 
     def test_input_data(self):
         """Check widget's data with data on the input"""
         self.assertEqual(self.widget.data, None)
-        self.send_signal("Data", self.data)
+        self.send_signal("数据(Data)", self.data)
         self.assertEqual(self.widget.data, self.data)
         self.wait_until_stop_blocking()
 
     def test_input_data_disconnect(self):
         """Check widget's data and model after disconnecting data from input"""
-        self.send_signal("Data", self.data)
+        self.send_signal("数据(Data)", self.data)
         self.assertEqual(self.widget.data, self.data)
         self.widget.apply_button.button.click()
         self.wait_until_stop_blocking()
-        self.send_signal("Data", None)
+        self.send_signal("数据(Data)", None)
         self.wait_until_stop_blocking()
         self.assertEqual(self.widget.data, None)
         self.assertIsNone(self.get_output(self.widget.Outputs.model))
@@ -269,19 +282,19 @@ class WidgetLearnerTestMixin:
     def test_input_data_learner_adequacy(self):
         """Check if error message is shown with inadequate data on input"""
         for inadequate in self.inadequate_dataset:
-            self.send_signal("Data", inadequate)
+            self.send_signal("数据(Data)", inadequate)
             self.widget.apply_button.button.click()
             self.wait_until_stop_blocking()
             self.assertTrue(self.widget.Error.data_error.is_shown())
         for valid in self.valid_datasets:
-            self.send_signal("Data", valid)
+            self.send_signal("数据(Data)", valid)
             self.wait_until_stop_blocking()
             self.assertFalse(self.widget.Error.data_error.is_shown())
 
     def test_input_preprocessor(self):
         """Check learner's preprocessors with an extra pp on input"""
         randomize = Randomize()
-        self.send_signal("Preprocessor", randomize)
+        self.send_signal("预处理器(Preprocessor)", randomize)
         self.assertEqual(
             randomize, self.widget.preprocessors,
             'Preprocessor not added to widget preprocessors')
@@ -294,7 +307,7 @@ class WidgetLearnerTestMixin:
     def test_input_preprocessors(self):
         """Check multiple preprocessors on input"""
         pp_list = PreprocessorList([Randomize(), RemoveNaNColumns()])
-        self.send_signal("Preprocessor", pp_list)
+        self.send_signal("预处理器(Preprocessor)", pp_list)
         self.widget.apply_button.button.click()
         self.wait_until_stop_blocking()
         self.assertEqual(
@@ -304,12 +317,12 @@ class WidgetLearnerTestMixin:
     def test_input_preprocessor_disconnect(self):
         """Check learner's preprocessors after disconnecting pp from input"""
         randomize = Randomize()
-        self.send_signal("Preprocessor", randomize)
+        self.send_signal("预处理器(Preprocessor)", randomize)
         self.widget.apply_button.button.click()
         self.wait_until_stop_blocking()
         self.assertEqual(randomize, self.widget.preprocessors)
 
-        self.send_signal("Preprocessor", None)
+        self.send_signal("预处理器(Preprocessor)", None)
         self.widget.apply_button.button.click()
         self.wait_until_stop_blocking()
         self.assertIsNone(self.widget.preprocessors,
@@ -317,10 +330,10 @@ class WidgetLearnerTestMixin:
 
     def test_output_learner(self):
         """Check if learner is on output after apply"""
-        initial = self.get_output("Learner")
+        initial = self.get_output("学习器(Learner)")
         self.assertIsNotNone(initial, "Does not initialize the learner output")
         self.widget.apply_button.button.click()
-        newlearner = self.get_output("Learner")
+        newlearner = self.get_output("学习器(Learner)")
         self.assertIsNot(initial, newlearner,
                          "Does not send a new learner instance on `Apply`.")
         self.assertIsNotNone(newlearner)
@@ -331,7 +344,7 @@ class WidgetLearnerTestMixin:
         self.assertIsNone(self.get_output(self.widget.Outputs.model))
         self.widget.apply_button.button.click()
         self.assertIsNone(self.get_output(self.widget.Outputs.model))
-        self.send_signal('Data', self.data)
+        self.send_signal("数据(Data)", self.data)
         self.widget.apply_button.button.click()
         self.wait_until_stop_blocking()
         model = self.get_output(self.widget.Outputs.model)
@@ -349,26 +362,27 @@ class WidgetLearnerTestMixin:
         self.widget.name_line_edit.setText(new_name)
         self.widget.apply_button.button.click()
         self.wait_until_stop_blocking()
-        self.assertEqual(self.get_output("Learner").name, new_name)
+        self.assertEqual(self.get_output("学习器(Learner)").name, new_name)
 
     def test_output_model_name(self):
         """Check if model's name properly changes"""
         new_name = "Model Name"
         self.widget.name_line_edit.setText(new_name)
-        self.send_signal("Data", self.data)
+        self.send_signal("数据(Data)", self.data)
         self.widget.apply_button.button.click()
         self.wait_until_stop_blocking()
         self.assertEqual(self.get_output(self.widget.Outputs.model).name, new_name)
 
     def test_output_model_picklable(self):
         """Check if model can be pickled"""
-        self.send_signal("Data", self.data)
+        self.send_signal("数据(Data)", self.data)
         self.widget.apply_button.button.click()
         self.wait_until_stop_blocking()
         model = self.get_output(self.widget.Outputs.model)
         pickle.dumps(model)
 
-    def _get_param_value(self, learner, param):
+    @staticmethod
+    def _get_param_value(learner, param):
         if isinstance(learner, Fitter):
             # Both is just a was to indicate to the tests, fitters don't
             # actually support this
@@ -384,7 +398,7 @@ class WidgetLearnerTestMixin:
         """Check if learner's parameters are set to default (widget's) values
         """
         for dataset in self.valid_datasets:
-            self.send_signal("Data", dataset)
+            self.send_signal("数据(Data)", dataset)
             self.widget.apply_button.button.click()
             self.wait_until_stop_blocking()
             for parameter in self.parameters:
@@ -399,7 +413,7 @@ class WidgetLearnerTestMixin:
         # Test params on every valid dataset, since some attributes may apply
         # to only certain problem types
         for dataset in self.valid_datasets:
-            self.send_signal("Data", dataset)
+            self.send_signal("数据(Data)", dataset)
             self.wait_until_stop_blocking()
 
             for parameter in self.parameters:
@@ -420,7 +434,7 @@ class WidgetLearnerTestMixin:
                     self.assertEqual(
                         param, value,
                         "Mismatching setting for parameter '%s'" % parameter)
-                    param = self._get_param_value(self.get_output("Learner"), parameter)
+                    param = self._get_param_value(self.get_output("学习器(Learner)"), parameter)
                     self.assertEqual(
                         param, value,
                         "Mismatching setting for parameter '%s'" % parameter)
@@ -436,7 +450,7 @@ class WidgetLearnerTestMixin:
     def test_params_trigger_settings_changed(self):
         """Check that the learner gets updated whenever a param is changed."""
         for dataset in self.valid_datasets:
-            self.send_signal("Data", dataset)
+            self.send_signal("数据(Data)", dataset)
             self.wait_until_stop_blocking()
 
             for parameter in self.parameters:
@@ -470,6 +484,14 @@ class WidgetLearnerTestMixin:
                  data.domain.has_continuous_class) or
                 (parameter.problem_type == "both"))
 
+    def test_send_report(self, timeout=DEFAULT_TIMEOUT):
+        """Test report"""
+        self.send_signal(self.widget.Inputs.data, self.data)
+        self.widget.report_button.click()
+        self.wait_until_finished(timeout=timeout)
+        self.send_signal(self.widget.Inputs.data, None)
+        self.widget.report_button.click()
+
 
 class WidgetOutputsTestMixin:
     """Class for widget's outputs testing.
@@ -488,9 +510,10 @@ class WidgetOutputsTestMixin:
     _compare_selected_annotated_domains.
     """
 
-    def init(self):
+    def init(self, same_table_attributes=True):
         self.data = Table("iris")
         self.same_input_output_domain = True
+        self.same_table_attributes = same_table_attributes
 
     def test_outputs(self, timeout=DEFAULT_TIMEOUT):
         self.send_signal(self.signal_name, self.signal_data)
@@ -498,38 +521,40 @@ class WidgetOutputsTestMixin:
         self.wait_until_finished(timeout=timeout)
 
         # check selected data output
-        self.assertIsNone(self.get_output("Selected Data"))
+        self.assertIsNone(self.get_output("选定的数据(Selected Data)"))
 
         # check annotated data output
         feature_name = ANNOTATED_DATA_FEATURE_NAME
-        annotated = self.get_output(ANNOTATED_DATA_SIGNAL_NAME)
+        annotated = self.get_output(ANNOTATED_DATA_SIGNAL_Chinese_NAME)
         self.assertEqual(0, np.sum([i[feature_name] for i in annotated]))
 
         # select data instances
         selected_indices = self._select_data()
 
         # check selected data output
-        selected = self.get_output("Selected Data")
+        selected = self.get_output("选定的数据(Selected Data)")
         n_sel, n_attr = len(selected), len(self.data.domain.attributes)
         self.assertGreater(n_sel, 0)
         self.assertEqual(selected.domain == self.data.domain,
                          self.same_input_output_domain)
         np.testing.assert_array_equal(selected.X[:, :n_attr],
                                       self.data.X[selected_indices])
-        self.assertEqual(selected.attributes, self.data.attributes)
+        if self.same_table_attributes:
+            self.assertEqual(selected.attributes, self.data.attributes)
 
         # check annotated data output
-        annotated = self.get_output(ANNOTATED_DATA_SIGNAL_NAME)
+        annotated = self.get_output(ANNOTATED_DATA_SIGNAL_Chinese_NAME)
         self.assertEqual(n_sel, np.sum([i[feature_name] for i in annotated]))
-        self.assertEqual(annotated.attributes, self.data.attributes)
+        if self.same_table_attributes:
+            self.assertEqual(annotated.attributes, self.data.attributes)
 
         # compare selected and annotated data domains
         self._compare_selected_annotated_domains(selected, annotated)
 
         # check output when data is removed
         self.send_signal(self.signal_name, None)
-        self.assertIsNone(self.get_output("Selected Data"))
-        self.assertIsNone(self.get_output(ANNOTATED_DATA_SIGNAL_NAME))
+        self.assertIsNone(self.get_output("选定的数据(Selected Data)"))
+        self.assertIsNone(self.get_output(ANNOTATED_DATA_SIGNAL_Chinese_NAME))
 
     def _select_data(self):
         raise NotImplementedError("Subclasses should implement select_data")
@@ -670,11 +695,9 @@ class ProjectionWidgetTestMixin:
 
     def test_dragging_tooltip(self):
         """Dragging tooltip depends on data being jittered"""
-        text = self.widget.graph.tiptexts[0]
+        text = QTextDocumentFragment.fromHtml(self.widget.graph.tiptexts[0]).toPlainText()
         self.send_signal(self.widget.Inputs.data, Table("heart_disease"))
         self.assertEqual(self.widget.graph.tip_textitem.toPlainText(), text)
-        self.widget.graph.controls.jitter_size.setValue(1)
-        self.assertGreater(self.widget.graph.tip_textitem.toPlainText(), text)
 
     def test_sparse_data(self, timeout=DEFAULT_TIMEOUT):
         """Test widget for sparse data"""
@@ -736,7 +759,7 @@ class ProjectionWidgetTestMixin:
         hidden_var1.attributes["hidden"] = True
         hidden_var2 = ContinuousVariable("c2")
         hidden_var2.attributes["hidden"] = True
-        class_vars = [DiscreteVariable("cls", values=["a", "b"])]
+        class_vars = [DiscreteVariable("cls", values=("a", "b"))]
         table = Table(Domain([hidden_var1, hidden_var2], class_vars),
                       np.array([[0., 1.], [2., 3.]]),
                       np.array([[0.], [1.]]))
@@ -746,8 +769,12 @@ class ProjectionWidgetTestMixin:
 
     def test_in_out_summary(self, timeout=DEFAULT_TIMEOUT):
         info = self.widget.info
-        self.assertEqual(info._StateInfo__input_summary.brief, "")
-        self.assertEqual(info._StateInfo__output_summary.brief, "")
+        self.assertEqual(info._StateInfo__input_summary.brief, "-")
+        self.assertEqual(info._StateInfo__output_summary.brief, "-")
+        self.assertIn(info._StateInfo__input_summary.details,
+                      ["", "No data on input"])
+        self.assertIn(info._StateInfo__output_summary.details,
+                      ["", "No data on output"])
 
         self.send_signal(self.widget.Inputs.data, self.data)
         self.assertTrue(
@@ -758,10 +785,84 @@ class ProjectionWidgetTestMixin:
         self.assertEqual(info._StateInfo__input_summary.brief,
                          str(len(self.data)))
         self.assertEqual(info._StateInfo__output_summary.brief, str(len(ind)))
+        self.assertGreater(info._StateInfo__input_summary.details, "")
+        self.assertGreater(info._StateInfo__output_summary.details, "")
+        self.assertNotIn(info._StateInfo__input_summary.details,
+                         ["", "No data on input"])
+        self.assertNotIn(info._StateInfo__output_summary.details,
+                         ["", "No data on output"])
 
         self.send_signal(self.widget.Inputs.data, None)
-        self.assertEqual(info._StateInfo__input_summary.brief, "")
-        self.assertEqual(info._StateInfo__output_summary.brief, "")
+        self.assertEqual(info._StateInfo__input_summary.brief, "-")
+        self.assertEqual(info._StateInfo__output_summary.brief, "-")
+        self.assertIn(info._StateInfo__input_summary.details,
+                      ["", "No data on input"])
+        self.assertIn(info._StateInfo__output_summary.details,
+                      ["", "No data on output"])
+
+    def test_visual_settings(self, timeout=DEFAULT_TIMEOUT):
+        graph = self.widget.graph
+        font = QFont()
+        font.setItalic(True)
+        font.setFamily("Helvetica")
+
+        self.send_signal(self.widget.Inputs.data, self.data)
+        self.wait_until_finished(timeout=timeout)
+
+        key, value = ("Fonts", "Font family", "Font family"), "Helvetica"
+        self.widget.set_visual_settings(key, value)
+
+        key, value = ("Fonts", "Title", "Font size"), 20
+        self.widget.set_visual_settings(key, value)
+        key, value = ("Fonts", "Title", "Italic"), True
+        self.widget.set_visual_settings(key, value)
+        font.setPointSize(20)
+        self.assertFontEqual(graph.parameter_setter.title_item.item.font(), font)
+
+        key, value = ("Fonts", "Label", "Font size"), 10
+        self.widget.set_visual_settings(key, value)
+        key, value = ("Fonts", "Label", "Italic"), True
+        self.widget.set_visual_settings(key, value)
+        simulate.combobox_activate_item(self.widget.controls.attr_label,
+                                        self.data.domain[0].name)
+        font.setPointSize(10)
+        self.assertFontEqual(graph.labels[0].textItem.font(), font)
+
+        key, value = ("Fonts", "Categorical legend", "Font size"), 14
+        self.widget.set_visual_settings(key, value)
+        key, value = ("Fonts", "Categorical legend", "Italic"), True
+        self.widget.set_visual_settings(key, value)
+        font.setPointSize(14)
+        legend_item = list(graph.parameter_setter.cat_legend_items)[0]
+        self.assertFontEqual(legend_item[1].item.font(), font)
+
+        key, value = ("Fonts", "Numerical legend", "Font size"), 12
+        self.widget.set_visual_settings(key, value)
+        key, value = ("Fonts", "Numerical legend", "Italic"), True
+        self.widget.set_visual_settings(key, value)
+        simulate.combobox_activate_item(self.widget.controls.attr_color,
+                                        self.data.domain[0].name)
+        variables = self.data.domain.variables + self.data.domain.metas
+        discrete_var = next(
+            (x for x in variables if isinstance(x, DiscreteVariable)), None
+        )
+        if discrete_var:
+            # activate item only when discrete variable available
+            simulate.combobox_activate_item(
+                self.widget.controls.attr_shape, discrete_var.name
+            )
+        font.setPointSize(12)
+        self.assertFontEqual(graph.parameter_setter.num_legend.items[0][0].font, font)
+
+        key, value = ("Annotations", "Title", "Title"), "Foo"
+        self.widget.set_visual_settings(key, value)
+        self.assertEqual(graph.parameter_setter.title_item.item.toPlainText(), "Foo")
+        self.assertEqual(graph.parameter_setter.title_item.text, "Foo")
+
+    def assertFontEqual(self, font1, font2):
+        self.assertEqual(font1.family(), font2.family())
+        self.assertEqual(font1.pointSize(), font2.pointSize())
+        self.assertEqual(font1.italic(), font2.italic())
 
 
 class AnchorProjectionWidgetTestMixin(ProjectionWidgetTestMixin):
@@ -770,13 +871,13 @@ class AnchorProjectionWidgetTestMixin(ProjectionWidgetTestMixin):
         table.X[0] = np.nan
         self.send_signal(self.widget.Inputs.data, table)
         self.assertFalse(np.all(self.widget.valid_data))
-        output = self.get_output(ANNOTATED_DATA_SIGNAL_NAME)
+        output = self.get_output(ANNOTATED_DATA_SIGNAL_Chinese_NAME)
         embedding_mask = np.all(np.isnan(output.metas[:, :2]), axis=1)
         np.testing.assert_array_equal(~embedding_mask, self.widget.valid_data)
         # reload
         self.send_signal(self.widget.Inputs.data, table)
 
-    def test_sparse_data(self):
+    def test_sparse_data(self, timeout=DEFAULT_TIMEOUT):
         table = Table("iris")
         table.X = sp.csr_matrix(table.X)
         self.assertTrue(sp.issparse(table.X))
@@ -807,6 +908,24 @@ class AnchorProjectionWidgetTestMixin(ProjectionWidgetTestMixin):
         self.assertEqual(len(self.widget.graph.scatterplot_item.data), nvalid)
         np.testing.assert_equal(self.widget.graph.selection, selection)
 
+    def test_visual_settings(self, timeout=DEFAULT_TIMEOUT):
+        super().test_visual_settings(timeout)
+
+        graph = self.widget.graph
+        font = QFont()
+        font.setItalic(True)
+        font.setFamily("Helvetica")
+
+        key, value = ("Fonts", "Anchor", "Font size"), 10
+        self.widget.set_visual_settings(key, value)
+        key, value = ("Fonts", "Anchor", "Italic"), True
+        self.widget.set_visual_settings(key, value)
+        font.setPointSize(10)
+        for item in graph.anchor_items:
+            if isinstance(item, AnchorItem):
+                item = item._label
+            self.assertFontEqual(item.textItem.font(), font)
+
 
 class datasets:
     @staticmethod
@@ -873,7 +992,7 @@ class datasets:
             Domain(
                 [ContinuousVariable("a"),
                  ContinuousVariable("b"),
-                 DiscreteVariable("c", values=["y", "n"])]
+                 DiscreteVariable("c", values=("y", "n"))]
             ),
             list(zip(
                 [42.48, 16.84, 15.23, 23.8],
