@@ -21,6 +21,7 @@ from Orange.widgets.utils.annotated_data import \
 from Orange.widgets.utils.concurrent import ThreadExecutor, FutureSetWatcher
 from Orange.widgets.utils.sql import check_sql_input
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.widget import Input, Output
 
 
@@ -101,7 +102,8 @@ class OWKMeans(widget.OWWidget):
     description = "基于轮廓质量估计的 K 均值聚类算法"
     icon = "icons/KMeans.svg"
     priority = 2100
-    keywords = ["kmeans", "clustering"]
+    keywords = ["kmeans", "clustering", 'kjunzhi']
+    category = 'unsupervised'
 
     class Inputs:
         data = Input("数据(Data)", Table, replaces=['Data'])
@@ -166,6 +168,9 @@ class OWKMeans(widget.OWWidget):
         self.__executor = ThreadExecutor(parent=self)
         self.__task = None  # type: Optional[Task]
 
+        self._set_input_summary()
+        self._set_output_summary(None)
+
         layout = QGridLayout()
         bg = gui.radioButtonsInBox(
             self.controlArea, self, "optimize_k", orientation=layout,
@@ -281,11 +286,23 @@ class OWKMeans(widget.OWWidget):
         if k > len(data):
             raise NotEnoughData()
 
-        return KMeans(
+        model = KMeans(
             n_clusters=k, init=init, n_init=n_init, max_iter=max_iter,
-            random_state=random_state
+            random_state=random_state, preprocessors=[]
         ).get_model(data)
 
+        if data.X.shape[0] <= SILHOUETTE_MAX_SAMPLES:
+            model.silhouette_samples = silhouette_samples(data.X, model.labels)
+            model.silhouette = np.mean(model.silhouette_samples)
+        else:
+            model.silhouette_samples = None
+            model.silhouette = \
+                silhouette_score(data.X, model.labels,
+                                 sample_size=SILHOUETTE_MAX_SAMPLES,
+                                 random_state=RANDOM_STATE)
+
+        return model
+
     @Slot(int, int)
     def __progress_changed(self, n, d):
         assert QThread.currentThread() is self.thread()
@@ -337,9 +354,10 @@ class OWKMeans(widget.OWWidget):
     def __launch_tasks(self, ks):
         # type: (List[int]) -> None
         """Execute clustering in separate threads for all given ks."""
+        preprocessed_data = self.preproces(self.data)
         futures = [self.__executor.submit(
             self._compute_clustering,
-            data=Normalize()(self.data) if self.normalize else self.data,
+            data=preprocessed_data,
             k=k,
             init=self.INIT_METHODS[self.smart_init][1],
             n_init=self.n_init,
@@ -442,9 +460,8 @@ class OWKMeans(widget.OWWidget):
             self.commit()
 
     def update_results(self):
-        scores = [mk if isinstance(mk, str) else silhouette_score(
-            self.preproces(self.data).X, mk.labels) for mk in (
-                self.clusterings[k] for k in range(self.k_from, self.k_to + 1))]
+        scores = [mk if isinstance(mk, str) else mk.silhouette for mk in
+                  (self.clusterings[k] for k in range(self.k_from, self.k_to + 1))]
         best_row = max(
             range(len(scores)), default=0,
             key=lambda x: 0 if isinstance(scores[x], str) else scores[x]
@@ -478,11 +495,6 @@ class OWKMeans(widget.OWWidget):
             data = preprocessor(data)
         return data
 
-    def samples_scores(self, clust_ids):
-        d = self.preproces(self.data)
-        return np.arctan(
-            silhouette_samples(d.X, clust_ids)) / np.pi + 0.5
-
     def send_data(self):
         if self.optimize_k:
             row = self.selected_row()
@@ -492,6 +504,7 @@ class OWKMeans(widget.OWWidget):
 
         km = self.clusterings.get(k)
         if self.data is None or km is None or isinstance(km, str):
+            self._set_output_summary(None)
             self.Outputs.annotated_data.send(None)
             self.Outputs.centroids.send(None)
             return
@@ -504,9 +517,9 @@ class OWKMeans(widget.OWWidget):
         clust_ids = km.labels
         silhouette_var = ContinuousVariable(
             get_unique_names(domain, "Silhouette"))
-        if len(self.data) <= SILHOUETTE_MAX_SAMPLES:
+        if km.silhouette_samples is not None:
             self.Warning.no_silhouettes.clear()
-            scores = self.samples_scores(clust_ids)
+            scores = np.arctan(km.silhouette_samples) / np.pi + 0.5
             clust_scores = []
             for i in range(km.k):
                 in_clust = clust_ids == i
@@ -525,10 +538,11 @@ class OWKMeans(widget.OWWidget):
         new_table.get_column_view(cluster_var)[0][:] = clust_ids
         new_table.get_column_view(silhouette_var)[0][:] = scores
 
+        domain_attributes = set(domain.attributes)
         centroid_attributes = [
             attr.compute_value.variable
             if isinstance(attr.compute_value, ReplaceUnknowns)
-            and attr.compute_value.variable in domain.attributes
+            and attr.compute_value.variable in domain_attributes
             else attr
             for attr in km.domain.attributes]
         centroid_domain = add_columns(
@@ -545,6 +559,7 @@ class OWKMeans(widget.OWWidget):
         else:
             centroids.name = f"{self.data.name} centroids"
 
+        self._set_output_summary(new_table)
         self.Outputs.annotated_data.send(new_table)
         self.Outputs.centroids.send(centroids)
 
@@ -553,6 +568,7 @@ class OWKMeans(widget.OWWidget):
     def set_data(self, data):
         self.data, old_data = data, self.data
         self.selection = None
+        self._set_input_summary()
 
         # Do not needlessly recluster the data if X hasn't changed
         if old_data and self.data and array_equal(self.data.X, old_data.X):
@@ -561,6 +577,16 @@ class OWKMeans(widget.OWWidget):
         else:
             self.invalidate(unconditional=True)
 
+    def _set_input_summary(self):
+        summary = len(self.data) if self.data else self.info.NoInput
+        details = format_summary_details(self.data) if self.data else ""
+        self.info.set_input_summary(summary, details)
+
+    def _set_output_summary(self, output):
+        summary = len(output) if output else self.info.NoOutput
+        details = format_summary_details(output) if output else ""
+        self.info.set_output_summary(summary, details)
+
     def send_report(self):
         # False positives (Setting is not recognized as int)
         # pylint: disable=invalid-sequence-index
