@@ -21,13 +21,13 @@ from Orange.data.util import get_unique_names, array_equal
 from Orange import preprocess
 from Orange.projection import PCA
 from Orange.widgets import widget, gui, report
-from Orange.widgets.settings import DomainContextHandler, ContextSetting, \
-    Setting
+from Orange.widgets.settings import Setting
 from Orange.widgets.utils.annotated_data import add_columns, \
     ANNOTATED_DATA_SIGNAL_Chinese_NAME
 from Orange.widgets.utils.concurrent import FutureWatcher
 from Orange.widgets.utils.signals import Input, Output
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.widget import Msg
 
 try:
@@ -42,7 +42,7 @@ _MAX_K_NEIGBOURS = 200
 _DEFAULT_K_NEIGHBORS = 30
 
 
-METRICS = [("欧几里德", "l2"), ("曼哈顿", "l1")]
+METRICS = [("欧几里德", "l2"), ("曼哈顿", "l1"), ("余弦", 'cosine')]
 
 
 class OWLouvainClustering(widget.OWWidget):
@@ -50,10 +50,12 @@ class OWLouvainClustering(widget.OWWidget):
     description = "检测最近邻居网络或者邻居的社区"
     icon = "icons/LouvainClustering.svg"
     priority = 2110
+    category = 'unsupervised'
+
+    settings_version = 2
 
     want_main_area = False
-
-    settingsHandler = DomainContextHandler()
+    resizing_enabled = False
 
     class Inputs:
         data = Input("数据(Data)", Table, default=True, replaces=['Data'])
@@ -63,12 +65,12 @@ class OWLouvainClustering(widget.OWWidget):
         if Network is not None:
             graph = Output("网络(Network)", Network, replaces=['Network'])
 
-    apply_pca = ContextSetting(True)
-    pca_components = ContextSetting(_DEFAULT_PCA_COMPONENTS)
-    normalize = ContextSetting(True)
-    metric_idx = ContextSetting(0)
-    k_neighbors = ContextSetting(_DEFAULT_K_NEIGHBORS)
-    resolution = ContextSetting(1.)
+    apply_pca = Setting(True)
+    pca_components = Setting(_DEFAULT_PCA_COMPONENTS)
+    normalize = Setting(True)
+    metric_idx = Setting(0)
+    k_neighbors = Setting(_DEFAULT_K_NEIGHBORS)
+    resolution = Setting(1.)
     auto_commit = Setting(False)
 
     class Information(widget.OWWidget.Information):
@@ -100,6 +102,8 @@ class OWLouvainClustering(widget.OWWidget):
         # Set up UI
         info_box = gui.vBox(self.controlArea, "信息")
         self.info_label = gui.widgetLabel(info_box, "没有输入数据。")  # type: QLabel
+        self.info.set_input_summary(self.info.NoInput)
+        self.info.set_output_summary(self.info.NoOutput)
 
         preprocessing_box = gui.vBox(self.controlArea, "预处理")
         self.normalize_cbx = gui.checkBox(
@@ -121,12 +125,12 @@ class OWLouvainClustering(widget.OWWidget):
             graph_box, self, "metric_idx", label="距离度量",
             items=[m[0] for m in METRICS], callback=self._invalidate_graph,
             orientation=Qt.Horizontal,
-        )  # type: gui.OrangeComboBox
+        )
         self.k_neighbors_spin = gui.spin(
             graph_box, self, "k_neighbors", minv=1, maxv=_MAX_K_NEIGBOURS,
             label="k 邻近", controlWidth=80, alignment=Qt.AlignRight,
             callback=self._invalidate_graph,
-        )  # type: gui.SpinBoxWFocusOut
+        )
         self.resolution_spin = gui.hSlider(
             graph_box, self, "resolution", minValue=0, maxValue=5., step=1e-1,
             label="分辨率", intOnly=False, labelFormat="%.1f",
@@ -387,6 +391,11 @@ class OWLouvainClustering(widget.OWWidget):
         new_domain = add_columns(domain, metas=[cluster_var])
         new_table = self.data.transform(new_domain)
         new_table.get_column_view(cluster_var)[0][:] = new_partition
+
+        summary = len(new_table) if new_table else self.info.NoOutput
+        details = format_summary_details(new_table) if new_table else ""
+        self.info.set_output_summary(summary, details)
+
         self.Outputs.annotated_data.send(new_table)
 
         if Network is not None:
@@ -399,11 +408,10 @@ class OWLouvainClustering(widget.OWWidget):
 
     @Inputs.data
     def set_data(self, data):
-        self.closeContext()
         self.Error.clear()
 
         prev_data, self.data = self.data, data
-        self.openContext(self.data)
+        self._set_input_summary()
         # Make sure to properly enable/disable slider based on `apply_pca` setting
         self.controls.pca_components.setEnabled(self.apply_pca)
 
@@ -414,6 +422,7 @@ class OWLouvainClustering(widget.OWWidget):
 
         self.cancel()
         # Clear the outputs
+        self.info.set_output_summary(self.info.NoOutput)
         self.Outputs.annotated_data.send(None)
         if Network is not None:
             self.Outputs.graph.send(None)
@@ -440,6 +449,11 @@ class OWLouvainClustering(widget.OWWidget):
 
         self.commit()
 
+    def _set_input_summary(self):
+        summary = len(self.data) if self.data else self.info.NoInput
+        details = format_summary_details(self.data) if self.data else ""
+        self.info.set_input_summary(summary, details)
+
     def clear(self):
         self.__cancel_task(wait=False)
         self.preprocessed_data = None
@@ -470,6 +484,20 @@ class OWLouvainClustering(widget.OWWidget):
             ("Resolution", self.resolution),
         ))
 
+    @classmethod
+    def migrate_settings(cls, settings, version):
+        if version < 2 and "context_settings" in settings:
+            try:
+                current_context = settings["context_settings"][0]
+                for n in ['apply_pca', 'k_neighbors', 'metric_idx',
+                          'normalize', 'pca_components', 'resolution']:
+                    if n in current_context.values:
+                        settings[n] = current_context.values[n][0]
+            except:  # pylint: disable=bare-except
+                pass
+            finally:
+                del settings["context_settings"]
+
 
 class TaskState(QObject):
 
