@@ -33,7 +33,7 @@ class TestOWtSNE(WidgetTest, ProjectionWidgetTestMixin, WidgetOutputsTestMixin):
         WidgetOutputsTestMixin.init(cls)
         cls.same_input_output_domain = False
 
-        cls.signal_name = "Data"
+        cls.signal_name = "数据(Data)"
         cls.signal_data = cls.data
 
     def setUp(self):
@@ -46,7 +46,7 @@ class TestOWtSNE(WidgetTest, ProjectionWidgetTestMixin, WidgetOutputsTestMixin):
 
         self.widget = self.create_widget(OWtSNE, stored_settings={"multiscale": False})
 
-        self.class_var = DiscreteVariable("Stage name", values=["STG1", "STG2"])
+        self.class_var = DiscreteVariable("Stage name", values=("STG1", "STG2"))
         self.attributes = [ContinuousVariable("GeneName" + str(i)) for i in range(5)]
         self.domain = Domain(self.attributes, class_vars=self.class_var)
         self.empty_domain = Domain([], class_vars=self.class_var)
@@ -55,7 +55,7 @@ class TestOWtSNE(WidgetTest, ProjectionWidgetTestMixin, WidgetOutputsTestMixin):
         # Some tests may not wait for the widget to finish, and the patched
         # methods might be unpatched before the widget finishes, resulting in
         # a very confusing crash.
-        self.widget.cancel()
+        self.widget.onDeleteWidget()
         try:
             self.restore_mocked_functions()
         # If `restore_mocked_functions` was called in the test itself, stopping
@@ -150,6 +150,7 @@ class TestOWtSNE(WidgetTest, ProjectionWidgetTestMixin, WidgetOutputsTestMixin):
         self.send_signal(w.Inputs.data, self.data, widget=w)
         self.assertTrue(w.controls.multiscale.isChecked())
         self.assertFalse(w.perplexity_spin.isEnabled())
+        w.onDeleteWidget()
 
     def test_normalize_data(self):
         # Normalization should be checked by default
