@@ -1,4 +1,5 @@
 import time
+import warnings
 from itertools import chain
 from typing import List
 from functools import singledispatch
@@ -14,7 +15,8 @@ from AnyQt.QtWidgets import (
     QSizePolicy, QDesktopWidget,
 )
 from AnyQt.QtGui import QColor, QPainter, QFont, QPen, QBrush
-from AnyQt.QtCore import Qt, QRectF, QSize
+from AnyQt.QtCore import Qt, QRectF, QSize, QPropertyAnimation, QObject, \
+    pyqtProperty
 
 from Orange.data import Table, Domain, DiscreteVariable, ContinuousVariable
 from Orange.statistics.util import nanmin, nanmax, nanmean, unique
@@ -25,6 +27,7 @@ from Orange.classification.logistic_regression import \
 from Orange.widgets.settings import Setting, ContextSetting, \
     ClassValuesContextHandler
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.widget import OWWidget, Msg, Input, Output, AttributeList
 from Orange.widgets import gui
 
@@ -63,6 +66,9 @@ class MovableToolTip(QLabel):
         super().show()
 
 
+DOT_COLOR = QColor(170, 220, 255, 255)
+
+
 class DotItem(QGraphicsEllipseItem):
     TOOLTIP_STYLE = """ul {margin-top: 1px; margin-bottom: 1px;}"""
     TOOLTIP_TEMPLATE = """<html><head><style type="text/css">{}</style>
@@ -77,7 +83,7 @@ class DotItem(QGraphicsEllipseItem):
         self._offset = offset
         self.setPos(0, - radius / 2)
         self.setFlag(QGraphicsItem.ItemIsMovable)
-        self.setBrush(QColor(170, 220, 255, 255))
+        self.setBrush(DOT_COLOR)
         self.setPen(QPen(QBrush(QColor(20, 130, 250, 255)), 2))
         self.setZValue(100)
         self.tool_tip = MovableToolTip()
@@ -145,6 +151,7 @@ class MovableDotItem(DotItem):
         self._total_dot = None
         self._probs_dot = None
         self._vertical_line = None
+        self._mousePressFunc = None
 
     @property
     def vertical_line(self):
@@ -182,7 +189,16 @@ class MovableDotItem(DotItem):
         self._probs_dot = dot
         self._probs_dot.movable_dot_items.append(self)
 
+    def hookOnMousePress(self, func):
+        self._mousePressFunc = func
+
+    def unhookOnMousePress(self):
+        self._mousePressFunc = None
+
     def mousePressEvent(self, event):
+        if self._mousePressFunc:
+            self._mousePressFunc()
+            self._mousePressFunc = None
         self.tool_tip.show(event.screenPos(), self.get_tooltip_text(), False)
         self._x = event.pos().x()
         self.setBrush(QColor(50, 180, 250, 255))
@@ -241,6 +257,54 @@ class DiscreteMovableDotItem(MovableDotItem):
                 (self.tooltip_labels[i].replace("<", "&lt;"), abs(1 - p1))]
 
 
+class GraphicsColorAnimator(QObject):
+    @pyqtProperty(QColor)
+    def brushColor(self):
+        return self.__brushColor
+
+    @brushColor.setter
+    def brushColor(self, value):
+        self.__brushColor = value
+        for item in self.__items:
+            item.setBrush(value)
+
+    def __init__(self, parent, duration, keyValues):
+        super().__init__(parent)
+        self.__items = []
+        self.__defaultColor = defaultColor = keyValues[0][1]
+        self.__brushColor = defaultColor
+
+        self.__animation = QPropertyAnimation(self, b'brushColor', self)
+        self.__animation.setStartValue(defaultColor)
+        self.__animation.setEndValue(defaultColor)
+        self.__animation.setDuration(duration)
+        self.__animation.setKeyValues(keyValues)
+        self.__animation.setLoopCount(-1)
+
+    def setGraphicsItems(self, items):
+        if self.__animation.state() == QPropertyAnimation.Running:
+            self.__animation.stop()
+        self.__items = items
+        for item in items:
+            item.hookOnMousePress(self.stop)
+
+    def start(self):
+        self.__animation.start()
+
+    def stop(self):
+        if self.__animation.state() != QPropertyAnimation.Running:
+            return
+        self.__animation.stop()
+        for item in self.__items:
+            item.setBrush(self.__defaultColor)
+
+    def clear(self):
+        for item in self.__items:
+            item.unhookOnMousePress()
+        self.__items = []
+
+
+
 class ContinuousItemMixin:
     def get_tooltip_text(self):
         return self.TOOLTIP_TEMPLATE.format(
@@ -587,7 +651,8 @@ class OWNomogram(OWWidget):
     description = " 朴素贝叶斯和逻辑回归分类器可视化的列线图。"
     icon = "icons/Nomogram.svg"
     priority = 2000
-    keywords = []
+    keywords = ['liexiantu']
+    category = 'visualize'
 
     class Inputs:
         classifier = Input("分类器(Classifier)", Model, replaces=['Classifier'])
@@ -639,11 +704,13 @@ class OWNomogram(OWWidget):
         self.old_target_class_index = self.target_class_index
         self.repaint = False
 
+        self.info.set_input_summary(self.info.NoInput)
+
         # GUI
         box = gui.vBox(self.controlArea, "目标类别")
         self.class_combo = gui.comboBox(
             box, self, "target_class_index", callback=self._class_combo_changed,
-            contentsLength=12)
+            contentsLength=12, searchable=True)
         self.norm_check = gui.checkBox(
             box, self, "normalize_probabilities", "归一化概率",
             hidden=True, callback=self.update_scene,
@@ -740,6 +807,17 @@ class OWNomogram(OWWidget):
         for view in (top_view, mid_view, bottom_view):
             self.mainArea.layout().addWidget(view)
 
+        self.dot_animator = GraphicsColorAnimator(
+            self, 3000,
+            [
+                (0.9, DOT_COLOR),
+                (0.925, DOT_COLOR.lighter(115)),
+                (0.95, DOT_COLOR),
+                (0.975, DOT_COLOR.lighter(115)),
+                (1.0, DOT_COLOR)
+            ]
+        )
+
     def _class_combo_changed(self):
         with np.errstate(invalid='ignore'):
             coeffs = [np.nan_to_num(p[self.target_class_index] /
@@ -762,7 +840,10 @@ class OWNomogram(OWWidget):
         self.norm_check.setHidden(True)
         self.cont_feature_dim_combo.setEnabled(True)
         if self.domain is not None:
-            self.class_combo.addItems(self.domain.class_vars[0].values)
+            values = self.domain.class_vars[0].values
+            if values:
+                self.class_combo.addItems(values)
+                self.target_class_index = 0
             if len(self.domain.attributes) > self.MAX_N_ATTRS:
                 self.display_index = 1
             if len(self.domain.class_vars[0].values) > 2:
@@ -783,6 +864,9 @@ class OWNomogram(OWWidget):
     @Inputs.data
     def set_data(self, data):
         self.instances = data
+        summary = len(data) if data else self.info.NoInput
+        details = format_summary_details(data) if data else ""
+        self.info.set_input_summary(summary, details)
         self.feature_marker_values = []
         self.set_feature_marker_values()
         self.update_scene()
@@ -800,7 +884,6 @@ class OWNomogram(OWWidget):
         self.calculate_log_odds_ratios()
         self.calculate_log_reg_coefficients()
         self.update_controls()
-        self.target_class_index = 0
         self.openContext(self.domain.class_var if self.domain is not None
                          else None)
         self.points = self.log_odds_ratios or self.log_reg_coeffs
@@ -847,7 +930,7 @@ class OWNomogram(OWWidget):
         if len(self.domain.class_var.values) == 2:
             self.b0 = np.hstack((self.b0 * (-1), self.b0))
             coeffs = np.vstack((coeffs * (-1), coeffs))
-        self.log_reg_coeffs = [coeffs[:, ranges[i]] for i in range(len(attrs))]
+        self.log_reg_coeffs = [coeffs[:, r] for r in ranges]
         self.log_reg_coeffs_orig = self.log_reg_coeffs.copy()
 
         min_values = nanmin(self.data.X, axis=0)
@@ -859,7 +942,7 @@ class OWNomogram(OWWidget):
                 coef = self.log_reg_coeffs[i]
                 self.log_reg_coeffs[i] = np.hstack((coef * min_t, coef * max_t))
                 self.log_reg_cont_data_extremes.append(
-                    [sorted([min_t, max_t], reverse=(c < 0)) for c in coef])
+                    [sorted([min_t, max_t], reverse=(c < 0)) for c in coef.flat])
             else:
                 self.log_reg_cont_data_extremes.append([None])
 
@@ -911,6 +994,11 @@ class OWNomogram(OWWidget):
             item.dot.probs_dot = probs_item.dot
             item.dot.vertical_line = self.hidden_vertical_line
 
+        self.dot_animator.setGraphicsItems(
+            [item.dot for item in self.feature_items.values()]
+        )
+        self.dot_animator.start()
+
         self.nomogram = nomogram = NomogramItem()
         nomogram.add_items([nomogram_head, self.nomogram_main, nomogram_foot])
         self.scene.addItem(nomogram)
@@ -1184,6 +1272,7 @@ class OWNomogram(OWWidget):
         self.nomogram_main = None
         self.vertical_line = None
         self.hidden_vertical_line = None
+        self.dot_animator.clear()
         self.scene.clear()
 
     def send_report(self):
@@ -1244,7 +1333,11 @@ class OWNomogram(OWWidget):
                sorted_coefficients[i] * sorted_values[i] * (1 - k)
 
     def reset_settings(self):
-        self._reset_settings()
+        with warnings.catch_warnings():
+            # setting target_class_index will trigger this innocent warning
+            warnings.filterwarnings(
+                "ignore", "combo box 'target_class_index' is empty")
+            self._reset_settings()
         self.update_scene()
 
 
